#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house
_bhukthi_house_list = [0,3,6,9,1,4,7,10,2,5,8,11]
_bhukthi_exempt_list_1 = [2,9]
_bhukthi_exempt_list_2 = [1,5,10,11] 
dhasa_adhipathi_dict = {1:1,2:2,3:9,5:20,4:18,0:20,6:50,'L':12} 
def get_dhasa_bhukthi(dob,tob,place,divisional_chart_factor=1,chart_method=1,years=1,months=1,sixty_hours=1,
                      include_antardhasa=True,mahadhasa_lord_has_no_antardhasa=True,
                      antardhasa_option1=False,antardhasa_option2=False):
    """
        provides Naisargika dhasa bhukthi for a given date in julian day (includes birth time)
        @param dob: Date Struct (year,month,day)
        @param tob: time tuple (h,m,s) 
        @param place: Place as tuple (place name, latitude, longitude, timezone)
        @param divisional_chart_factor Default=1 
            1=Raasi, 9=Navamsa. See const.division_chart_factors for options
        @param chart_method: Default=1, various chart methods available for each div chart. See charts module
        @param years: Yearly chart. number of years from date of birth
        @param months: Monthly chart. number of months from date of birth
        @param sixty_hours: 60-hour chart. number of 60 hours from date of birth
        @param include_antardhasa: True (include) False (exclude) antardhasa (Default=True)
        @param mahadhasa_lord_has_no_antardhasa=True => Mahadhasa lord has no antardhasa. Default=True
        @param antardhasa_option1=True => Planets in 3rd and 10th from dasa lord have no antardhasa. Default=False
        @param antardhasa_option2=True => Planets in 2nd,6th,11th and 12th from dasa lord have no antardhasa. Default=False
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    bhukthi_house_list = _bhukthi_house_list
    if antardhasa_option1:
        bhukthi_house_list = [p for p in bhukthi_house_list if p not in _bhukthi_exempt_list_1]
    if antardhasa_option2:
        bhukthi_house_list = [p for p in bhukthi_house_list if p not in _bhukthi_exempt_list_2]
    start_jd = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(start_jd, place, divisional_chart_factor=divisional_chart_factor, 
                                               chart_method=chart_method,years=years, months=months, 
                                               sixty_hours=sixty_hours)[:8]# Ignore Rahu onwards
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions)
    dhasa_lords = list(dhasa_adhipathi_dict.keys())
    dhasa_info = []
    for dhasa_lord in dhasa_lords:
        duration = dhasa_adhipathi_dict[dhasa_lord]
        lord_house = planet_positions[dhasa_lord+1][1][0] if dhasa_lord != const._ascendant_symbol else planet_positions[0][1][0] 
        bhukthis = [h_to_p[(h+lord_house)%12].split('/') for h in bhukthi_house_list if h_to_p[(h+lord_house)%12] != '']
        bhukthis = utils.flatten_list(bhukthis)
        [bhukthis.remove(p) for p in ['L','7','8'] if p in bhukthis]
        if mahadhasa_lord_has_no_antardhasa and dhasa_lord != const._ascendant_symbol and str(dhasa_lord) in bhukthis:
            bhukthis.remove(str(dhasa_lord))   
        bhukthis = list(map(int,bhukthis))
        if include_antardhasa:
            """ 
                TODO: Antardasa period should be based on weights per planets' placement 
                See: https://srath.com/jyoti%E1%B9%A3a/dasa/naisargika-dasha/ for details
            """
            dd = round(duration/len(bhukthis),2)
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,duration))
            start_jd += duration * const.sidereal_year
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.naisargika_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
    This is an attempt to create horoscope based surya sidhantha meant/true position calculations
    Reference: Indian Astronomy - An Introduction - S. Balachandra Rao
    WORK STILL IN PROGRESS - NOT WORKING FOR SOME PLANETS YET
"""
import math
from jhora import utils, const
from jhora.panchanga import drik as drik
import swisseph as swe
from jhora.horoscope.chart import charts
from jhora.panchanga.vratha import pradosham_dates

sind = lambda x : math.sin(math.radians(x)) 
cosd = lambda x : math.cos(math.radians(x))
to_dms = lambda x: utils.to_dms(x,is_lat_long='plong')
ahargana_khanda_khaadyaka = lambda jd: jd - 1964031
#ahargana_graha_laghavam = lambda jd: jd - 1687850
mandaphala_of_sun = 0.0
def kali_ahargana(jd):
    """ TODO: CHECK: Should this be int or float? """
    kad = int(jd - 588464.54) # (jd - 588466)
    wday = int(kad) % 7
    wdayjd = drik.vaara(jd)
    winc = (wdayjd - 5 - wday)
    wdayc = (wday + winc + 5) % 7
    assert wdayc==wdayjd
    return kad+winc
def _mean_solar_longitude(jd):
    mean_daily_motion_sun_at_kali = const.mean_revolutions_sun_kali / const.civil_days_in_mahayuga * 360
    kan = kali_ahargana(jd)
    mean_longitude_sun = (kan * mean_daily_motion_sun_at_kali) %360
    return mean_longitude_sun,utils.to_dms(mean_longitude_sun,is_lat_long='plong')
def _mean_lunar_longitude(jd):
    mean_daily_motion_moon_at_kali = const.mean_revolutions_moon_kali / const.civil_days_in_mahayuga * 360
    kan = kali_ahargana(jd)
    mean_longitude_moon = (kan * mean_daily_motion_moon_at_kali) %360
    return mean_longitude_moon,utils.to_dms(mean_longitude_moon,is_lat_long='plong')
def _mean_lunar_apogee_longitude(jd):
    mean_daily_motion_moon_apogee = const.madocca_revolutions[const._MOON] / const.civil_days_in_mahayuga * 360
    kan = kali_ahargana(jd)
    mean_longitude_moon_apogee = (kan * mean_daily_motion_moon_apogee) %360 + const.manodcca_positions_at_kali[const._MOON]
    return mean_longitude_moon_apogee,utils.to_dms(mean_longitude_moon_apogee,is_lat_long='plong')
def _mean_rahu_longitude(jd):
    mean_daily_motion_rahu_at_kali = const.mean_revolutions_rahu_kali / const.civil_days_in_mahayuga * 360
    kan = kali_ahargana(jd)
    mean_longitude_rahu = 180.0 - (kan * mean_daily_motion_rahu_at_kali) %360
    return mean_longitude_rahu,utils.to_dms(mean_longitude_rahu,is_lat_long='plong')
def _mean_ketu_longitude(jd):
    mean_longitude_ketu = (180.0 + _mean_rahu_longitude(jd)[0]) % 360 
    return mean_longitude_ketu,utils.to_dms(mean_longitude_ketu,is_lat_long='plong')
def _planet_mean_longitude(jd, place,planet):
    """ TODO: Mars not matching close to Drik """
    """ RESET Planet's mean longitude"""
    p_id = drik.planet_list.index(planet)
    const.planet_mean_longitudes[planet] = 0.0 
    mean_revolutions = const.planet_mean_revolutions_at_kali[planet]
    mean_daily_motion = round(mean_revolutions / const.civil_days_in_mahayuga * 360,7) #const.daily_mean_motions[planet] #
    kan = kali_ahargana(jd)
    mean_longitude = ((kan * mean_daily_motion) + 360) %360
    #print('planet,kan,mean_revolutions,mean_daily_motion,mean_longitude',p_id,kan,mean_revolutions,
    #      mean_daily_motion,const.daily_mean_motions[planet],mean_longitude,utils.to_dms(mean_longitude,is_lat_long='plong'))
    if planet in [const._RAHU, const._KETU]:
        mean_longitude = 180.0 - mean_longitude
    if planet == const._KETU:
        mean_longitude = (180.0 + mean_longitude) % 360
    #print('mean long',kan,planet,mean_daily_motion,mean_revolutions,mean_longitude)
    """ Apply Corrections """
    dc = _desantara_correction(place, planet)
    corrected_long = (mean_longitude+dc+360.0)%360
    #print(p_id,'mean longitude after DC',corrected_long,utils.to_dms(corrected_long,is_lat_long='plong')) 
    const.planet_mean_longitudes[planet] = corrected_long   
    return corrected_long
def _desantara_correction(place:drik.Place,planet):
    p_id = drik.planet_list.index(planet)
    planet_daily_motion = const.daily_mean_motions[planet]
    plong = place.longitude
    ulong = const.ujjain_lat_long[1]
    dc = (ulong - plong)/360.0 * planet_daily_motion
    #print('_desantara_correction',p_id,dc,utils.to_dms(dc,is_lat_long='plong'))
    return dc
def bhujantara_correction(planet,mandaphala_of_sun):
    """ TODO: To use Planet's true motion """
    p_id = drik.planet_list.index(planet)
    bc = const.daily_mean_motions[planet]*mandaphala_of_sun/360
    #print(p_id,'bhujantara_correction',mandaphala_of_sun*60,const.daily_mean_motions[planet]*60,bc,utils.to_dms(bc,is_lat_long='plong'))
    return bc
def ascendant_new(jd, place:drik.Place, sun_long):
    """ TODO NOT WORKING STILL UNDER TESTING """
    _,lat,_,_ = place
    R = 3438
    eps = math.radians(24.0)
    def _delination(lat): # radians
        delination = [11.733998794908114, 20.624646006223887, 24.0]
        td = [R/6* math.tan(math.radians(lat))* math.tan(math.radians(d)) for d in delination]
        td = [-td[0]] + [-(y-x) for x,y in zip(td,td[1:])]
        td1 = td.copy()
        td1.reverse()
        td += [-d1 for d1 in td1]
        td2 = td.copy()
        td2.reverse()
        td += [d1 for d1 in td2]
        return td
    td = _delination(lat)
    lanka_rising_durations = [278,299,323,323,299,278,278,299,323,323,299,278]
    place_rising_durations = [d1+d2 for d1,d2 in zip(lanka_rising_durations,td)]
    sun_rise = drik.sunrise(jd, place)
    jd_sunrise = sun_rise[-1]
    y, m, d,_  = utils.jd_to_gregorian(jd)
    jd_utc = utils.gregorian_to_jd(drik.Date(y, m, d))
    #print(jd,jd_sunrise,sun_rise[1],jd_utc)
    time_from_sunrise_vinadi = (jd-jd_sunrise)*24/2.5*60
    sun_long_rem = sun_long % 30
    sun_long_rem_vinadi = sun_long_rem * 60.0
    sun_long_raasi = int(sun_long/30)
    #print('sun_long',sun_long,'sun_long_raasi',sun_long_raasi,'sun_long_rem',sun_long_rem)
    place_rising = place_rising_durations[sun_long_raasi]*sun_long_rem_vinadi/(30*60)
    #print('time_from_sunrise_vinadi',time_from_sunrise_vinadi,'sun_long_rem_vinadi',sun_long_rem_vinadi,'place_rising',place_rising)
    residue = time_from_sunrise_vinadi-place_rising
    #print('residue',residue)
    asc_long = residue/place_rising_durations[(sun_long_raasi+1)%12]*30
    #print('asc_long',asc_long)
    return int(asc_long/30),asc_long%30
def ascendant(jd, place:drik.Place, sun_long):
    """ TODO NOT WORKING STILL UNDER TESTING """
    _,lat,_,_ = place
    R = 3438
    eps = math.radians(24.0)
    def _delination(lat): # radians
        delination = [11.733998794908114, 20.624646006223887, 24.0]
        td = [R/6* math.tan(math.radians(lat))* math.tan(math.radians(d)) for d in delination]
        td = [-td[0]] + [-(y-x) for x,y in zip(td,td[1:])]
        td1 = td.copy()
        td1.reverse()
        td += [-d1 for d1 in td1]
        td2 = td.copy()
        td2.reverse()
        td += [d1 for d1 in td2]
        return td
    td = _delination(lat)
    lanka_rising_durations = [278,299,323,323,299,278,278,299,323,323,299,278]
    place_rising_durations = [d1+d2 for d1,d2 in zip(lanka_rising_durations,td)]
    #print(lanka_rising_durations)
    #print(td)
    #print(place_rising_durations)
    sun_long_rem = sun_long % 30
    sun_long_rem_vinadi = sun_long_rem * 60.0
    sun_long_raasi = int(sun_long/30)
    place_rising = place_rising_durations[sun_long_raasi]*sun_long_rem_vinadi/(30*60)
    srt = utils.from_dms_str_to_dms(drik.sunrise(jd, place)[1])
    ud = drik.udhayadhi_nazhikai((10,34,0), srt)[1]
    asc_long = (ud * 60. - sun_long_rem_vinadi)/place_rising_durations[sun_long_raasi]*30.0
    asc_rasi = int(asc_long / 30)
    asc_coordinates = asc_long % 30 
    if const._TROPICAL_MODE:
        flags = swe.FLG_SWIEPH
    else:
        flags = swe.FLG_SIDEREAL
        drik.set_ayanamsa_mode(drik._ayanamsa_mode,drik._ayanamsa_value,jd) # needed for swe.houses_ex()
    nak_no,paadha_no,_ = drik.nakshatra_pada(asc_long)
    return [const._ascendant_symbol,[asc_rasi,asc_coordinates]]#,nak_no,paadha_no]
def _mandaphala_planet_new(jd,planet):
    """ Mandaphala using just periphery and mandocca """
    pass
def _true_daily_motion_planet(jd,planet):
    planet_mean_long = _planet_mean_longitude(jd,place,planet)
    p_id = drik.planet_list.index(planet)
    if planet in [const._RAHU, const._KETU]:
        return 0.0
    kan = kali_ahargana(jd)
    mandocca_mean_revolutions = const.madocca_revolutions[planet]
    planet_mandocca_at_kali = const.manodcca_positions_at_kali[planet]
    mandocca_motion = (kan / (const.civil_days_in_mahayuga) * mandocca_mean_revolutions*360) %360 # Degrees
    #print(p_id,'kan,mandocca_motion',kan,mandocca_mean_revolutions,mandocca_motion,utils.to_dms(mandocca_motion,is_lat_long='plong'))
    planet_mandocca = planet_mandocca_at_kali + mandocca_motion
    #print(p_id,'planet_mandocca_at_kali,mandocca_motion,planet_mandocca',planet_mandocca_at_kali,mandocca_motion,planet_mandocca)
    #planet_mandocca_anomaly = (planet_mandocca + 360.0 - planet_mean_long) %360 
    #print(p_id,'planet_mandocca',utils.to_dms(planet_mandocca,is_lat_long='plong'))
    #print(p_id,'planet mean long, planet_anomaly',utils.to_dms(planet_mean_long,is_lat_long='plong'),utils.to_dms(planet_mandocca_anomaly,is_lat_long='plong'))
    planet_mean_motion = const.daily_mean_motions[planet] 
    mandakendra = (planet_mandocca - planet_mean_long + 360 ) % 360
    planet_mandocca_anomaly = mandakendra
    #print(p_id,'mandakendra',utils.to_dms(mandakendra,is_lat_long='plong'))
    mandakendra_sign = 1.0
    if mandakendra > 180.0:
        mandakendra_sign = -1.0
    " Mandaphala - Equarion of Center"
    if planet in [const._SUN, const._MOON]:
        #planet_mandaphala_periphery = const.planet_mandaphala_periphery_modern[planet]
        #corrected_periphery = planet_mandaphala_periphery - (1.0/3.0)*abs(sind(mandakendra))
        Po,Pe = const.planet_mandaphala_periphery_modern[planet]
        corrected_periphery = Pe - (Pe-Po) * abs(sind(mandakendra))
        if planet == const._MOON:
            print(p_id,'planet_mean_motion before',planet_mean_motion)
            planet_mean_motion = planet_mean_motion - const.moon_apogee_mean_motion
            print(p_id,'planet_mean_motion after',planet_mean_motion)
        #print(p_id,'corrected_periphery',corrected_periphery)
        rectified_periphery = (const.mandakendrajya_indian_sine_radius/360.0) * corrected_periphery
        #print(p_id,'rectified_periphery',rectified_periphery) 
        tab_sine_diff = abs(225*0.991335735*cosd(1.024764846*(180.0-planet_mandocca_anomaly-3.75)))
        #print(p_id,'tab_sine_diff',tab_sine_diff)
        " You should use true motion here"
        planet_true_motion_correction = corrected_periphery*planet_mean_motion*tab_sine_diff/(360*225)
        planet_true_motion = planet_mean_motion + mandakendra_sign * planet_true_motion_correction
    else:
        Po,Pe = const.planet_mandaphala_periphery_modern[planet]
        corrected_periphery = Pe - (Pe-Po) * abs(sind(mandakendra))
        #print(p_id,'mandaphala corrected_periphery',Po,Pe,corrected_periphery)
        mandaphala_correction = const.mandakendrajya_indian_sine_radius/360.0 * corrected_periphery * sind(mandakendra)
        planet_true_motion_correction = corrected_periphery*planet_mean_motion*tab_sine_diff/(360*225)
        planet_true_motion = planet_mean_motion + mandakendra_sign * planet_true_motion_correction
    return planet_true_motion
    
def _mandaphala_planet(jd,planet,planet_mean_long):
    p_id = drik.planet_list.index(planet)
    if planet in [const._RAHU, const._KETU]:
        return 0.0
    kan = kali_ahargana(jd)
    mandocca_mean_revolutions = const.madocca_revolutions[planet]
    planet_mandocca_at_kali = const.manodcca_positions_at_kali[planet]
    mandocca_motion = (kan / (const.civil_days_in_mahayuga) * mandocca_mean_revolutions*360) %360 # Degrees
    #print(p_id,'kan,mandocca_motion',kan,mandocca_mean_revolutions,mandocca_motion,utils.to_dms(mandocca_motion,is_lat_long='plong'))
    planet_mandocca = planet_mandocca_at_kali + mandocca_motion
    #print(p_id,'planet_mandocca_at_kali,mandocca_motion,planet_mandocca',planet_mandocca_at_kali,mandocca_motion,planet_mandocca)
    #planet_mandocca_anomaly = (planet_mandocca + 360.0 - planet_mean_long) %360 
    #print(p_id,'planet_mandocca',utils.to_dms(planet_mandocca,is_lat_long='plong'))
    #print(p_id,'planet mean long, planet_anomaly',utils.to_dms(planet_mean_long,is_lat_long='plong'),utils.to_dms(planet_mandocca_anomaly,is_lat_long='plong'))
    planet_mean_motion = const.daily_mean_motions[planet] 
    mandakendra = (planet_mandocca - planet_mean_long + 360 ) % 360
    planet_mandocca_anomaly = mandakendra
    #print(p_id,'mandakendra',utils.to_dms(mandakendra,is_lat_long='plong'))
    mandakendra_sign = 1.0
    if mandakendra > 180.0:
        mandakendra_sign = -1.0
    " Mandaphala - Equarion of Center"
    if planet in [const._SUN, const._MOON]:
        #planet_mandaphala_periphery = const.planet_mandaphala_periphery_modern[planet]
        #corrected_periphery = planet_mandaphala_periphery - (1.0/3.0)*abs(sind(mandakendra))
        Po,Pe = const.planet_mandaphala_periphery_modern[planet]
        corrected_periphery = Pe - (Pe-Po) * abs(sind(mandakendra))
        if planet == const._MOON:
            #print(p_id,'planet_mean_motion before',planet_mean_motion)
            planet_mean_motion = planet_mean_motion - const.moon_apogee_mean_motion
            #print(p_id,'planet_mean_motion after',planet_mean_motion)
        #print(p_id,'corrected_periphery',corrected_periphery)
        rectified_periphery = (const.mandakendrajya_indian_sine_radius/360.0) * corrected_periphery
        #print(p_id,'rectified_periphery',rectified_periphery) 
        tab_sine_diff = abs(225*0.991335735*cosd(1.024764846*(180.0-planet_mandocca_anomaly-3.75)))
        #print(p_id,'tab_sine_diff',tab_sine_diff)
        " You should use true motion here"
        planet_true_motion_correction = corrected_periphery*planet_mean_motion*tab_sine_diff/(360*225)
        planet_true_motion = planet_mean_motion + mandakendra_sign * planet_true_motion_correction
        #print(p_id,'planet_true_motion',planet_mean_motion,planet_true_motion_correction,corrected_periphery,tab_sine_diff,utils.to_dms(planet_true_motion,is_lat_long='plong')) 
        mandaphala_correction = mandakendra_sign * rectified_periphery*sind(mandakendra)
    else:
        Po,Pe = const.planet_mandaphala_periphery_modern[planet]
        corrected_periphery = Pe - (Pe-Po) * abs(sind(mandakendra))
        #print(p_id,'mandaphala corrected_periphery',Po,Pe,corrected_periphery)
        mandaphala_correction = const.mandakendrajya_indian_sine_radius/360.0 * corrected_periphery * sind(mandakendra)
    """ TODO Check sine inverse is required for MPH """
    #mandaphala_correction = math.asin(mandaphala_correction*math.pi/180.0)
    #print(p_id,'mandaphala_correction',mandaphala_correction)
    return mandaphala_correction
def _true_longitude_after_sighra_correction(jd,place,planet,planet_mean_long,mandaphala_correction):
    p_id = drik.planet_list.index(planet)
    MP = planet_mean_long
    if planet in [const._SUN, const._MOON, const._RAHU, const._KETU]:
        return 0.0
    p_id = drik.planet_list.index(planet)
    def _get_sighra_anamoly(jd,planet):
        if const.planet_mean_longitudes[const._SUN]==0.0:
            const.planet_mean_longitudes[const._SUN] = _mean_solar_longitude(jd)[0] #_planet_mean_longitude(jd, place, const._SUN)
        if planet in [const._MERCURY, const._VENUS]:
            m = (planet_mean_long - const.planet_mean_longitudes[const._SUN]+360)%360
            #print(p_id,'Sighra anomaly = Planets sighrocca - mean sun',to_dms(planet_mean_long),'-',to_dms(const.planet_mean_longitudes[const._SUN]),'=',to_dms(m))                  
        else:
            m = (const.planet_mean_longitudes[const._SUN] - planet_mean_long+360)%360 #const.planet_mean_longitudes[planet]+360)%360
            #print(p_id,'Sighra anomaly = Mean sun - mean planet',to_dms(const.planet_mean_longitudes[const._SUN]),'-', to_dms(const.planet_mean_longitudes[planet]),'=',to_dms(m))
        return m
    def _get_sighra_correction(jd,planet,p_m_l):
        m = _get_sighra_anamoly(jd, planet)
        """
        p_id = drik.planet_list.index(planet)
        if const.planet_mean_longitudes[const._SUN]==0.0:
            const.planet_mean_longitudes[const._SUN] = _mean_solar_longitude(jd)[0] #_planet_mean_longitude(jd, place, const._SUN)
        if planet in [const._MERCURY, const._VENUS]:
            m = (p_m_l - const.planet_mean_longitudes[const._SUN]+360)%360
            print(p_id,'Sighra anomaly = Planets sighrocca - mean sun',to_dms(p_m_l),'-',to_dms(const.planet_mean_longitudes[const._SUN]),'=',to_dms(m))                  
        else:
            m = (const.planet_mean_longitudes[const._SUN] - p_m_l+360)%360 #const.planet_mean_longitudes[planet]+360)%360
            #print(p_id,'Sighra anomaly = Mean sun - mean planet',to_dms(const.planet_mean_longitudes[const._SUN]),'-', to_dms(const.planet_mean_longitudes[planet]),'=',to_dms(m))
        """
        #print(p_id,'sighra anamoly - m',m)
        sign = 1.0
        if m > 90. and m < 270.0:
            sign = -1.0
        """ Calculate corrected sighra periphery """
        Po,Pe = const.planet_sighra_peripheries[planet]
        P = Pe - (Pe-Po) * abs(sind(m))
        r = P/360.0
        R = const.mandakendrajya_indian_sine_radius * 60.0
        #print(p_id,'Po,Pe,P,r,sighra anomaly(m)',to_dms(Po),to_dms(Pe),to_dms(P),r,to_dms(m))
        dohphala = r * R * sind(m)
        kotiphala = r * R * cosd(m)
        sphutakoti = R + kotiphala
        sighrakarna = math.sqrt(sphutakoti*sphutakoti+dohphala*dohphala)
        rsinsighraphala = dohphala*R/sighrakarna
        #print(p_id,'dohphala,kotiphala,sphutakoti,sighrakarna,rsinsighraphala',dohphala,kotiphala,sphutakoti,sighrakarna,rsinsighraphala)
        sighraphala = math.asin(rsinsighraphala/R)*180.0/math.pi
        #print(p_id,'sighraphala',to_dms(sighraphala))
        return sighraphala
    SE1 = _get_sighra_correction(jd,planet,MP)
    P1 = MP + 0.5 * SE1
    #print(p_id,'first step _get_sighra_correction','MP,SE1,P1',MP,SE1,P1)
    ME1 = mandaphala_correction
    P2 = P1 + 0.5 * ME1
    #print(p_id,'2nd step','P1,ME1,P2',P1,ME1,P2)
    ME2 = _mandaphala_planet(jd,planet, P2)
    P3 = MP + ME2
    const.planet_mean_longitudes[planet] = P3
    #print(p_id,'3rd step _mandaphala_planet','MP,ME2,P3',MP,ME2,P3)
    SE2 = _get_sighra_correction(jd,planet,P3)
    P4 = P3 + SE2
    #print(p_id,'4th step _get_sighra_correction','P3,SE2,P4',P3,SE2,P4)
    return P4
def _planet_true_longitude(jd,place,planet,planet_mean_long):
    p_id = drik.planet_list.index(planet)
    if planet in [const._RAHU, const._KETU]:
        return planet_mean_long
    """ Correction for Equation of Center aka Mandaphala """
    mandaphala = _mandaphala_planet(jd,planet,planet_mean_long)
    if planet == const._SUN:
        global mandaphala_of_sun
        mandaphala_of_sun = mandaphala
    #print(p_id,'mandaphala',mandaphala,utils.to_dms(mandaphala,is_lat_long='plong'))
    mandaphala_corrected_longitude = planet_mean_long + mandaphala
    #print(p_id,'_planet_true_longitude',planet_mean_long,mandaphala_of_sun,mandaphala_corrected_longitude,utils.to_dms(mandaphala_corrected_longitude,is_lat_long='plong'))
    bcs = bhujantara_correction(planet, mandaphala_of_sun)
    mandaphala_corrected_longitude += bcs
    true_longitude = (mandaphala_corrected_longitude+360)%360
    #print(p_id,'planet true longitude',true_longitude)
    if planet in [const._SUN, const._MOON, const._RAHU, const._KETU]: 
        return true_longitude
    else:
        true_longitude = (_true_longitude_after_sighra_correction(jd,place,planet,planet_mean_long,mandaphala)+360)%360
        """ CRUDE FIX - INCORRECT - NO LOGIC
        if planet in [const._MERCURY]:#, const._VENUS]:
            true_longitude += const.manodcca_positions_at_kali[planet]
        elif planet in [const._VENUS]:#, const._VENUS]:
            true_longitude -= const.manodcca_positions_at_kali[planet]
        """
        return true_longitude
def planet_positions(jd,place:drik.Place):
    planet_positions_ss = []
    #planet_corrections = [0,90.0,0,-120.0,0,-180,0,0,0]
    #drik.set_ayanamsa_mode('SURYASIDDHANTA', ayanamsa_value=None, jd=jd)
    #ss_ayanamsa = drik.get_ayanamsa_value(jd)
    #print('SS Ayanamsa',ss_ayanamsa)
    planet_corrections = [0,0.0,0,0.0,0,0,0,0,0] # [x-ss_ayanamsa for x in [0,0.0,0,0.0,0,0,0,0,0]]
    for planet in drik.planet_list: #[const._SUN,const._MOON,const._SATURN]:#
        p_id = drik.planet_list.index(planet)
        mean_long = _planet_mean_longitude(jd,place,planet)
        #print(p_id,'mean longitude',planet,mean_long,utils.to_dms(mean_long, is_lat_long='plong'))
        corrected_long = (_planet_true_longitude(jd,place,planet, mean_long)+planet_corrections[p_id])%360.
        planet_positions_ss.append([p_id,[int(corrected_long/30),corrected_long%30]]    )
        #print(p_id,'true longitude',planet,corrected_long,utils.to_dms(corrected_long, is_lat_long='plong'))
    # Calculate ascendant and add to planet positions
    #asc = drik.ascendant(jd, place)
    sun_long = planet_positions_ss[0][1][0]*30+planet_positions_ss[0][1][1]
    asc = ascendant_new(jd, place, sun_long)
    planet_positions_ss = [[const._ascendant_symbol,[asc[0],asc[1]]]] + planet_positions_ss
    #print('planet_positions_ss',planet_positions_ss)
    return planet_positions_ss
def _lunar_evection(sun_mean_longitude, moon_mean_longitude):
    pass
def _declination_of_sun_1(jd):
    d = jd - 2451545.0
    mean_anamoly_sun = (357.529*0.98560028*d)%360
    mean_long_sun = (280.459 + 0.98564736 * d)%360
    sun_ecliptic_long = mean_long_sun + 1.915*sind(mean_anamoly_sun) + 0.020*sind(2*mean_anamoly_sun)
    obliquity_sun = 23.439 - 0.00000036 * d
    decl = math.asin(sind(obliquity_sun) * sind(sun_ecliptic_long))*180/math.pi
    #print(jd,mean_anamoly_sun,mean_long_sun,obliquity_sun,decl)
    return decl
def _declination_of_sun(jd):
    y,m,d,_ = utils.jd_to_gregorian(jd)
    jd_eq = utils.julian_day_number((y,3,21), (12,0,0))
    days = jd-jd_eq
    deli = (23+27/60.)*sind(360*days/365.25)
    #print(jd_eq,jd,days,deli)
    return deli
def sunrise_set(jd,place):
    decl = drik.declination_of_planets(jd, place)
    decl_sun = _declination_of_sun(jd)
    lat = place.latitude
    prad = math.pi/180.0
    sr_hrs = math.acos(-math.tan(lat*prad)*math.tan(decl_sun*prad))/prad/15.0
    srise = 12.0-sr_hrs
    sset = 12.0+sr_hrs
    return [srise,utils.to_dms(srise),sset,utils.to_dms(sset)]
def tithi(jd, place):
    pp = planet_positions(jd, place)
    sun_long = pp[1][1][0]*30+pp[1][1][1]
    moon_long = pp[2][1][0]*30+pp[2][1][1]
    l_diff = (moon_long+360-sun_long)%360
    _tithi = (l_diff/12)%30
    _tithi_no = math.ceil(_tithi)
    _td_left = _tithi_no*12-l_diff
    sun_dm = _true_daily_motion_planet(jd, const._SUN)
    moon_dm = _true_daily_motion_planet(jd, const._MOON)
    _th_left = _td_left/(moon_dm-sun_dm)*24.0
    _,_,_,h = utils.jd_to_gregorian(jd)
    _,_,_,fh = utils.jd_to_gregorian(jd+_th_left)
    fh +=h
    return[_tithi_no,utils.to_dms(fh)]
def nakshatra(jd,place):
    pp = planet_positions(jd, place)
    moon_long = pp[2][1][0]*30+pp[2][1][1]
    nak_no,padham_no,_ = drik.nakshatra_pada(moon_long)
    rem = (nak_no / 27 * 360.)-moon_long
    moon_dm = _true_daily_motion_planet(jd, const._MOON)
    _nak_left = rem/moon_dm*24.0
    _,_,_,h = utils.jd_to_gregorian(jd)
    _,_,_,fh = utils.jd_to_gregorian(jd+_nak_left)
    fh +=h
    return [nak_no,padham_no,utils.to_dms(fh)]
    
if __name__ == "__main__":
    #place = drik.Place('Ujjain',23.1765, 75.7885,5.5)
    #dob = drik.Date(-3101,1,22)
    #tob = (6,37,11)
    #dob = drik.Date(505,3,23)
    #dob = (0,0,1)
    #place = drik.Place('Bangalore',12.9716,77.5946,5.5)
    #dob = drik.Date(1970,3,22)
    #tob = (0,0,1)
    place = drik.Place('Chennai',13.0878,80.2785,5.5)
    dob = drik.Date(1996,12,7)
    tob = (10,34,0)
    jd = utils.julian_day_number(dob, tob)
    print(jd,'ahargana',kali_ahargana(jd))
    print(sunrise_set(jd, place),drik.sunrise(jd, place),drik.sunset(jd, place))
    print(tithi(jd, place),drik.tithi(jd, place))
    print(nakshatra(jd,place),drik.nakshatra(jd, place))
    exit()
    #"""
    planet_positions_ss = planet_positions(jd,place)
    sun_long = planet_positions_ss[1][1][0]*30+planet_positions_ss[1][1][1]
    print(ascendant_new(jd, place, sun_long))
    print(planet_positions_ss)
    planet_positions_drik = charts.rasi_chart(jd, place)
    print(planet_positions_drik)
    for p in range(len(planet_positions_ss)):
        p_long_ss = planet_positions_ss[p][1][0]*30+planet_positions_ss[p][1][1]
        p_long_drik = planet_positions_drik[p][1][0]*30+planet_positions_drik[p][1][1]
        p_diff = abs(p_long_drik-p_long_ss)
        print(planet_positions_drik[p][0],'drik-posn',p_long_drik,'ss-posn',p_long_ss,'diff',p_diff)
    sun_long = planet_positions_ss[1][1][0]*30+planet_positions_ss[1][1][1]
    print(sun_long,ascendant_new(jd, place, sun_long), drik.ascendant(jd, place))


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import utils, const
from jhora.horoscope.chart import charts
from jhora.panchanga import drik
def get_dhasa_bhukthi(dob,tob,place,divisional_chart_factor=1,chart_method=1,years=1,months=1,sixty_hours=1,
                      include_antardhasa=True):
    """
        provides Buddhi Gathi dhasa bhukthi for a given date in julian day (includes birth time)
        @param dob: Date Struct (year,month,day)
        @param tob: time tuple (h,m,s) 
        @param place: Place as tuple (place name, latitude, longitude, timezone)
        @param divisional_chart_factor Default=1 
            1=Raasi, 9=Navamsa. See const.division_chart_factors for options
        @param chart_method: default=1; various methods available for each div chart. see charts module. 
        @param years: Yearly chart. number of years from date of birth
        @param months: Monthly chart. number of months from date of birth
        @param sixty_hours: 60-hour chart. number of 60 hours from date of birth
        @param include_antardhasa: True (include) False (exclude) antardhasa (Default=True)
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, 
                                               divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                               years=years,months=months, sixty_hours=sixty_hours)
    h_to_p = utils.get_house_planet_list_from_planet_positions(planet_positions[1:])
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    planet_dict = {int(p):p_long for p,(_,p_long) in planet_positions[1:]}
    asc_house = p_to_h[const._ascendant_symbol]
    dhasa_progression = []
    h1 = 0
    start_jd = jd_at_dob
    for h in range(12):
        hs = (asc_house+3+h)%12
        # Get planets from the house
        if h_to_p[hs] == '': continue
        planets = list(map(int,h_to_p[hs].split('/')))
        d1 = {p:l for p,l in planet_dict.items() if p in planets}
        pl_new = [p for (p,_) in sorted(d1.items(), key=lambda item:item[1],reverse=True)]
        for pl in pl_new:
            durn = ((asc_house+h1+12) - p_to_h[pl])%12
            dhasa_progression.append((pl,durn))
            h1 += 1
    dhasa_bhukthi_info = []; dhasa_len = len(dhasa_progression); total_dhasa_duration = 0
    for dhasa_cycle in range(2):
        for dhasa in range(dhasa_len):
            dhasa_lord,dhasa_duration = dhasa_progression[dhasa]
            total_dhasa_duration += dhasa_duration
            if include_antardhasa:
                bhukthi_duration = dhasa_duration/dhasa_len
                for bhukthi in range(dhasa_len):
                    y,m,d,h = utils.jd_to_gregorian(start_jd)
                    dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                    bhukthi_lord = dhasa_progression[(dhasa+bhukthi)%dhasa_len][0]
                    dhasa_bhukthi_info.append((dhasa_lord,bhukthi_lord,dhasa_start,round(bhukthi_duration,2)))
                    start_jd += bhukthi_duration*const.sidereal_year
            else:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_bhukthi_info.append((dhasa_lord,dhasa_start,dhasa_duration))
                start_jd += dhasa_duration*const.sidereal_year
            if total_dhasa_duration >= const.human_life_span_for_narayana_dhasa:
                break
    return dhasa_bhukthi_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.buddhi_gathi_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import numpy as np
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import house, charts
""" TODO: Dhasa Progression does not seem to match with JHora """
def _get_dhasa_progression(planet_longitude):
    nakshatra,paadham,_ = drik.nakshatra_pada(planet_longitude)
    nakshatra -= 1
    paadham -= 1
    kalachakra_index = 0
    if nakshatra in const.savya_stars_1:
        kalachakra_index = 0
    elif nakshatra in const.savya_stars_2:
        kalachakra_index = 1
    elif nakshatra in const.apasavya_stars_1:
        kalachakra_index = 2
    else:
        kalachakra_index = 3
    dhasa_progression = const.kalachakra_rasis[kalachakra_index][paadham]
    dhasa_paramayush = const.kalachakra_paramayush[kalachakra_index][paadham]
    dhasa_duration = [const.kalachakra_dhasa_duration[r] for r in dhasa_progression]
    one_star = (360.0/27)
    one_paadha = (360.0 / 108)
    nak_start_long = nakshatra*one_star + paadham * one_paadha
    nak_travel_fraction = (planet_longitude-nak_start_long)/one_paadha
    dhasa_duration_cumulative = np.cumsum(dhasa_duration)
    paramayush_completed = nak_travel_fraction * dhasa_paramayush
    dhasa_index_at_birth = next(x[0] for x in enumerate(dhasa_duration_cumulative) if x[1] > paramayush_completed)
    dhasa_remaining_at_birth = dhasa_duration_cumulative[dhasa_index_at_birth]-paramayush_completed
    kalachakra_index_next = kalachakra_index
    paadham_next = (paadham+1)%4
    if paadham==3:
        if kalachakra_index == 0:
            kalachakra_index_next = 1
        elif kalachakra_index == 1:
            kalachakra_index_next = 0
        elif kalachakra_index == 2:
            kalachakra_index_next = 3
        elif kalachakra_index == 3:
            kalachakra_index_next = 2
    dhasa_progression = dhasa_progression[dhasa_index_at_birth:]+const.kalachakra_rasis[kalachakra_index_next][paadham_next][:dhasa_index_at_birth]
    dhasa_duration = [const.kalachakra_dhasa_duration[r] for r in dhasa_progression]
    dhasa_duration[0] = dhasa_remaining_at_birth
    dhasa_periods = []
    for i,dp in enumerate(dhasa_progression):
        ad = antardhasa(dhasa_index_at_birth,i, dhasa_paramayush, kalachakra_index_next, paadham)
        """
            Temporary Fix if ad = empty list
        """
        if len(ad)==0: ad=[dhasa_progression,[const.kalachakra_dhasa_duration[r] for r in dhasa_progression]]
        dhasa_periods.append([dp,ad,dhasa_duration[i]])
    return dhasa_periods
def antardhasa(dhasa_index_at_birth,dp_index,paramayush,kc_index,paadham):
    dp_begin = kc_index*9*4+paadham*9+dhasa_index_at_birth+dp_index
    antardhasa_progression=const.kalachakra_rasis_list[dp_begin:dp_begin+9]
    antardhasa_duration = [const.kalachakra_dhasa_duration[r] for r in antardhasa_progression]
    """ TODO: handle if above is empty list [] """
    if len(antardhasa_duration)==0:
        return []
    dhasa_duration = antardhasa_duration[0]
    antardhasa_fraction = dhasa_duration/sum(antardhasa_duration)
    antardhasa_duration = [(ad * antardhasa_fraction) for ad in antardhasa_duration]
    return [antardhasa_progression,antardhasa_duration]
def kalachakra_dhasa(planet_longitude,jd,include_antardhasa=True):
    """
        Kalachara Dhasa calculation
        @param planet_longitude: Longitude of planet (default=moon) at the time of Date/time of birth as float
        @param dob: Date of birth as tuple (year,month,day)
        @return: list of [dhasa_rasi,dhasa_rasi_start_date, dhasa_rasi_end_date,[abtadhasa_rasis],dhasa_rasi_duration]
        Example: [[7, '1946-12-2', '1955-12-2', [7, 8, 9, 10, 11, 0, 1, 2, 3, 4, 5, 6], 9], [8, '1955-12-2', '1964-12-2', [8, 9, 10, 11, 0, 1, 2, 3, 4, 5, 6, 7], 9], ...]
    """
    dhasa_periods = _get_dhasa_progression(planet_longitude)
    if len(dhasa_periods)==0:
        return []
    dhasa_start_jd = jd
    dp_new = []
    for dp in dhasa_periods:
        ds,ad,dd = dp
        #print('kalachakra dhasa values ds,ad,dd',ds,ad,dd)
        if include_antardhasa:
            for b in range(len(ad[0])):
                bhukthi_lord = ad[0][b]; bhukthi_duration = ad[1][b] 
                y,m,d,h = utils.jd_to_gregorian(dhasa_start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dp_new.append([ds,bhukthi_lord,dhasa_start,round(bhukthi_duration,2)])
                dhasa_start_jd += bhukthi_duration*const.sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(dhasa_start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dp_new.append([ds,dhasa_start,round(dd,2)])
        dhasa_duration_in_days = dd*const.sidereal_year
        dhasa_start_jd += dhasa_duration_in_days
    dhasa_periods = dp_new[:]
    return dhasa_periods
def get_dhasa_bhukthi(dob,tob,place,divisional_chart_factor=1,dhasa_starting_planet=1,
                      include_antardhasa=True,star_position_from_moon=1):
    """
        returns kalachakra dhasa bhukthi
        @param dob = Date of Birth as drik.Date tuple
        @param tob = Time of birth as tuple (h,m,s) 
        @param place: Place as tuple (place name, latitude, longitude, timezone) 
        @param star_position_from_moon: 
            1 => Default - moon
            4 => Kshema Star (4th constellation from moon)
            5 => Utpanna Star (5th constellation from moon)
            8 => Adhana Star (8th constellation from moon)
        @param dhasa_starting_planet 0=Sun 1=Moon(default)...8=Ketu, 'L'=Lagna
                                    M=Maandi, G=Gulika, T=Trisphuta, B=Bhindu, I=Indu, P=Pranapada
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
        
    """
    jd = utils.julian_day_number(dob, tob)
    from jhora.horoscope.chart import charts,sphuta
    _special_planets = ['M','G','T','I','B','I','P']
    planet_positions = charts.divisional_chart(jd, place,divisional_chart_factor=divisional_chart_factor)
    if dhasa_starting_planet in [*range(9)]:
        planet_long = planet_positions[dhasa_starting_planet+1][1][0]*30+planet_positions[dhasa_starting_planet+1][1][1]
    elif dhasa_starting_planet==const._ascendant_symbol:
        planet_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    elif dhasa_starting_planet.upper()=='M':
        mn = drik.maandi_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
        planet_long = mn[0]*30+mn[1]
    elif dhasa_starting_planet.upper()=='G':
        gl = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='B':
        gl = drik.bhrigu_bindhu_lagna(jd, place,divisional_chart_factor=divisional_chart_factor)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='I':
        gl = drik.indu_lagna(jd, place,divisional_chart_factor=divisional_chart_factor)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='P':
        gl = drik.pranapada_lagna(jd, place,divisional_chart_factor=divisional_chart_factor)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='T':
        sp = sphuta.tri_sphuta(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
        planet_long = sp[0]*30+sp[1]
    else:
        planet_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    if dhasa_starting_planet==1:
        one_star = (360 / 27.)        # 27 nakshatras span 360°
        planet_long += (star_position_from_moon-1)*one_star
    return kalachakra_dhasa(planet_long, jd,include_antardhasa=include_antardhasa)
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    utils.set_language('en')
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.kalachakra_dhasa_tests()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
year_duration = const.sidereal_year
""" Karana Based Chathuraaseethi Sama Dasa """

seed_lord = 0
dhasa_adhipathi_dict = const.karana_lords
dhasa_adhipathi_list = {k:12 for k in range(len(dhasa_adhipathi_dict))} # duration 12 years Total 84 years
count_direction = 1 # 1> base star to birth star zodiac -1> base star to birth star antizodiac
def _dhasa_adhipathi(karana_index):
    for key,(karana_list,durn) in dhasa_adhipathi_dict.items():
        if karana_index in karana_list:
            return key,durn 
def _next_adhipati(lord,dirn=1):
    """Returns next lord after `lord` in the adhipati_list"""
    current = list(dhasa_adhipathi_list.keys()).index(lord)
    next_lord = list(dhasa_adhipathi_list.keys())[((current + dirn) % len(dhasa_adhipathi_list))]
    return next_lord
def _maha_dhasa(nak):
    return [(_dhasa_lord, dhasa_adhipathi_list[_dhasa_lord]) for _dhasa_lord,_star_list in dhasa_adhipathi_dict.items() if nak in _star_list][0]
def _antardhasa(dhasa_lord,antardhasa_option=1):
    lord = dhasa_lord
    if antardhasa_option in [3,4]:
        lord = _next_adhipati(dhasa_lord, dirn=1) 
    elif antardhasa_option in [5,6]:
        lord = _next_adhipati(dhasa_lord, dirn=-1) 
    dirn = 1 if antardhasa_option in [1,3,5] else -1
    _bhukthis = []
    for _ in range(len(dhasa_adhipathi_list)):
        _bhukthis.append(lord)
        lord = _next_adhipati(lord,dirn)
    return _bhukthis
def _dhasa_start(jd,place):
    _,_,_,birth_time_hrs = utils.jd_to_gregorian(jd)
    _kar = drik.karana(jd, place)
    k_frac = utils.get_fraction(_kar[1], _kar[2], birth_time_hrs)
    lord,res = _dhasa_adhipathi(_kar[0])          # ruler of current nakshatra
    period_elapsed = (1-k_frac)*res*year_duration
    start_date = jd - period_elapsed      # so many days before current day
    return [lord, start_date,res]
def get_dhasa_bhukthi(dob,tob,place,include_antardhasa=True,use_tribhagi_variation=False,
                      divisional_chart_factor=1,chart_method=1,antardhasa_option=1):
    """
        provides karana chathuraaseethi sama dhasa bhukthi for a given date in julian day (includes birth time)
        @param dob: Date Struct (year,month,day)
        @param tob: time tuple (h,m,s) 
        @param place: Place as tuple (place name, latitude, longitude, timezone)
        @param use_tribhagi_variation: False (default), True means dhasa bhukthi duration in three phases 
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    _tribhagi_factor = 1.
    _dhasa_cycles = 1
    if use_tribhagi_variation:
        _tribhagi_factor = 1./3.; _dhasa_cycles = int(_dhasa_cycles/_tribhagi_factor)
    jd = utils.julian_day_number(dob, tob)
    dhasa_lord, start_jd,_ = _dhasa_start(jd,place)
    retval = []
    for _ in range(_dhasa_cycles):
        for _ in range(len(dhasa_adhipathi_list)):
            _dhasa_duration = round(dhasa_adhipathi_list[dhasa_lord]*_tribhagi_factor,2)
            if include_antardhasa:
                bhukthis = _antardhasa(dhasa_lord,antardhasa_option)
                _dhasa_duration /= len(bhukthis)
                for bhukthi_lord in bhukthis:
                    y,m,d,h = utils.jd_to_gregorian(start_jd)
                    dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                    retval.append((dhasa_lord,bhukthi_lord,dhasa_start,round(_dhasa_duration,2)))
                    start_jd += _dhasa_duration * year_duration
            else:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                retval.append((dhasa_lord,dhasa_start,_dhasa_duration))
                lord_duration = round(dhasa_adhipathi_list[dhasa_lord]*_tribhagi_factor,2)
                start_jd += lord_duration * year_duration
            dhasa_lord = _next_adhipati(dhasa_lord) # dirn=1 for dhasa sequence
    return retval
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.karana_chathuraseethi_sama_test()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house

year_duration = const.sidereal_year

def get_dhasa_antardhasa(dob,tob,place,divisional_chart_factor=1,chart_method=1,years=1,months=1,
                         sixty_hours=1,include_antardhasa=True):
    """
        provides karaka dhasa bhukthi for a given date in julian day (includes birth time)
        @param dob: Date Struct (year,month,day)
        @param tob: time tuple (h,m,s) 
        @param place: Place as tuple (place name, latitude, longitude, timezone)
        @param divisional_chart_factor Default=1 
            1=Raasi, 9=Navamsa. See const.division_chart_factors for options
        @param chart_method: Default=1, various chart methods available for each div chart. See charts module
        @param years: Yearly chart. number of years from date of birth
        @param months: Monthly chart. number of months from date of birth
        @param sixty_hours: 60-hour chart. number of 60 hours from date of birth
        @param include_antardhasa: True (include) False (exclude) antardhasa (Default=True)
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start] if include_antardhasa=True
        @return: a list of [dhasa_lord,dhasa_start] if include_antardhasa=False
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, 
                                               divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                               years=years,months=months, sixty_hours=sixty_hours)
    karakas = house.chara_karakas(planet_positions)
    asc_house = planet_positions[0][1][0]
    dhasa_info = []
    start_jd = jd_at_dob
    human_life_span = sum([(planet_positions[k+1][1][0] - asc_house +12)%12 for k in karakas])
    kl = len(karakas)
    for ki,k in enumerate(karakas):
        k_h = planet_positions[k+1][1][0]
        duration = (k_h - asc_house + 12)%12
        bhukthis = karakas[ki+1:kl]+karakas[0:ki+1]
        if include_antardhasa:
            for bhukthi_lord in bhukthis:
                b_h = planet_positions[bhukthi_lord+1][1][0]
                dd = (b_h - asc_house + 12)%12
                factor = dd *  duration / human_life_span
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((k,bhukthi_lord,dhasa_start,dd))
                start_jd += factor * year_duration
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((k,dhasa_start,duration))
            start_jd += duration * year_duration
    return dhasa_info
        

if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.karaka_dhasa_test()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house
from jhora.horoscope.dhasa.raasi import chara, sthira
""" Mahadasa match with JHora. Antardasa does not match with JHora """
sidereal_year = const.sidereal_year

def get_dhasa_antardhasa(dob,tob,place,divisional_chart_factor=1,years=1,months=1,sixty_hours=1,include_antardhasa=True):
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, 
                                               divisional_chart_factor=divisional_chart_factor, years=years, 
                                               months=months, sixty_hours=sixty_hours)
    asc_house = planet_positions[0][1][0]
    dhasa_seed = house.stronger_rasi_from_planet_positions(planet_positions, asc_house, (asc_house+6)%12)
    dhasa_lords = [(dhasa_seed+h)%12 for h in range(12)]
    if dhasa_seed in const.even_signs:
        dhasa_lords = [(dhasa_seed-h+12)%12 for h in range(12)]
    dhasa_info = []
    start_jd = jd_at_dob
    for dhasa_lord in dhasa_lords:
        duration = 0.5*(chara._dhasa_duration(planet_positions, dhasa_lord)+sthira._dhasa_duration(dhasa_lord))
        bhukthis = chara._antardhasa(dhasa_lords)# [(dhasa_lord+h)%12 for h in range(12)]
        if include_antardhasa:
            dd = duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,duration))
            start_jd += duration * sidereal_year
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.yogardha_dhasa_test()


from PyQt6.QtWidgets import QApplication, QMainWindow, QWidget, QLabel, QVBoxLayout, QGridLayout, QSizePolicy
from PyQt6.QtGui import QPainter, QColor,QFont, QPen
from PyQt6.QtCore import Qt, QRectF, QPoint, QPointF, QSize
import sys, math
from jhora import utils,const
from winnt import ACCESS_PROPERTY_GUID
"""
        TODO:
            Lagnam falls in abhijith
            Check get_2d_index or function to calculate planet-star mapping
            
"""

star_list = ["Asw","Bha","Kri","Roh","Mrig","Ardr","Puna","Push","Asre","Magh","PPha","UPha","Hast","Chit","Swat",
             "Visa","Anu","Jye","Mool","PSha","USha","Srav","Dhan","Sat","PBha","UBha","Rev","Abhi"]
get_2d_list_index = utils.get_2d_list_index
color_label = lambda label,color: '<span style=color:'+color+';>'+label+'</span>'
def drawRotatedText(painter, degrees, x, y, text):
    painter.save()
    painter.translate(x, y)
    painter.rotate(degrees)
    painter.drawText(0, 0, text)
    painter.restore()
class SapthaNaadi(QWidget):
    def __init__(self, planet_positions=[],planets_in_retrograde=[]):
        QWidget.__init__(self)
        self._xdim = 5; self._ydim = 7
        self._grid_label_values = [
            ['prachanda_str','pawan_str', 'dahan_str', 'soumya_str', 'neera_str', 'jala_str', 'amrit_str'],
            [3,4,5,6,7,8,9],[16,15,14,13,12,11,10],[17,18,19,20,21,22,23],[2,1,28,27,26,25,24,23]
        ]
        self._star_indices = {self._grid_label_values[i][j]:(i,j) for i in range(self._xdim) for j in range(self._ydim) if isinstance(self._grid_label_values[i][j],int)}
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
        self.createUI()
    def createUI(self):
        self.layout = QVBoxLayout()
        self.gridLayout = QGridLayout()
        self._grid_labels = [[QLabel('') for j in range(self._ydim)] for i in range(self._xdim)]
        for i in range(self._xdim):
            for j in range(self._ydim):
                #self._grid_labels[i][j].setText(self._grid_label_values[i][j])
                self._grid_labels[i][j].setStyleSheet("border: 1px solid black;")  # Optional: Add a border to each cell
                self.gridLayout.addWidget(self._grid_labels[i][j], i, j)
        self.layout.addLayout(self.gridLayout)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setContentsMargins(0,0,0,0)
        self.setLayout(self.layout)
        self._update_labels_with_stars_and_rasis()
        if len(self._planet_positions) > 0:
            self._update_with_planet_labels()
    def _update_with_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            r,c = self._star_indices[p_star]
            cell_value = self._grid_labels[r][c].text()+'<br>'+color_label(pstr+rstr,self._planet_color)
            self._grid_labels[r][c].setText(cell_value)
        self.update()
    def setData(self,planet_positions=[],planets_in_retrograde=[],label_font_size=8):
        if len(planet_positions) > 0:
            self._planet_positions = planet_positions
            self._update_with_planet_labels()
    def _update_labels_with_stars_and_rasis(self):
        for i in range(self._xdim):
            for j in range(self._ydim):
                cell_value = self._grid_label_values[i][j]
                star_cell = isinstance(cell_value,int)
                if star_cell:
                    cell_value = color_label(utils.NAKSHATRA_SHORT_LIST[cell_value-1],self._star_color)
                    self._grid_labels[i][j].setText(cell_value)
                    self._grid_labels[i][j].setAlignment(Qt.AlignmentFlag.AlignTop)
                else:
                    cell_value = color_label(utils.resource_strings[cell_value],self._text_color)
                    self._grid_labels[i][j].setText(cell_value)
                    self._grid_labels[i][j].setAlignment(Qt.AlignmentFlag.AlignCenter)
class PanchaShalaka(QWidget):
    def __init__(self,planet_positions=[],planets_in_retrograde=[],label_font_size=6):
        super().__init__()
        self.x_dim = 7; self.y_dim = 7; self.x_gap = 60; self.y_gap=60
        self.setGeometry(10, 10, 600, 600)
        self.setWindowTitle(utils.resource_strings['saptha_shalaka_str']+' '+utils.resource_strings['chakra_str'])
        self.lines = [((2,1),(2,7)),((3,1),(3,7)),((4,1),(4,7)),((5,1),(5,7)),((6,1),(6,7)),
                      ((1,2),(7,2)),((1,3),(7,3)),((1,4),(7,4)),((1,5),(7,5)),((1,6),(7,6)),
                      ((1,6.5),(6.5,1)),((1.5,7),(7,1.5)),((1.5,1),(7,6.5)),((1,1.5),(6.5,7))
                      ]
        self.rasi_labels = {
            1: (1,2), 2: (1, 1.5), 3: (1.5, 1), 4: (2,1), 5: (3,1),6:(4,1),7:(5,1), 8: (6,1), 9: (6.5,1),
            10: (7,1.5), 11: (7,2), 12: (7,3), 13:(7,4),14:(7,5), 15: (7, 6), 16: (7, 6.5), 17: (6.5,7), 18: (6,7),
            19: (5,7), 20:(4,7), 21:(3,7), 22: (2,7), 23: (1.5, 7),
            24: (1, 6.5), 25: (1, 6), 26: (1,5), 27: (1, 4), 28: (1, 3)
        }
        self._label_font_size = label_font_size
        self._base_star = 1
        self._update_with_star_labels()
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
    def _update_with_star_labels(self):
        _star_list = utils.NAKSHATRA_SHORT_LIST
        _abhijit_list = [_star_list[i] for i in const.abhijit_order_of_stars]
        _star_labels = {}
        for k,(x,y) in self.rasi_labels.items():
            nak = ((self._base_star-1)+(k-1))%28
            #self.rasi_labels[k] = (x,y,_abhijit_list[nak])
            _star_labels[nak+1] = (x,y,_abhijit_list[nak])
        self.rasi_labels = _star_labels.copy()
    def _update_with_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            if p_star > 21: p_star += 1
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            x,y,pl = self.rasi_labels[p_star]
            self.rasi_labels[p_star] = [x,y,pl+'\n'+pstr+rstr]        
    def setData(self,planet_positions,planets_in_retrograde=[],label_font_size=None):
        if label_font_size != None: self._label_font_size = label_font_size
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._update_with_planet_labels()
        self.createUI()
    def createUI(self):
        painter = QPainter(self)
        painter.setPen(QPen(Qt.GlobalColor.black, 2, Qt.PenStyle.SolidLine))
        x_gap = self.x_gap; y_gap = self.y_gap
        # Draw lines
        for line in self.lines:
            start, end = line
            painter.drawLine(int(start[0] * x_gap), int((start[1]) * y_gap), int(end[0] * x_gap), int((end[1]) * y_gap))

        # Draw text
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold); painter.setFont(font)
        for _, (x, y,star_planets) in self.rasi_labels.items():
            color = self._planet_color if '\n' in star_planets else self._star_color
            painter.setPen(QColor(color))
            painter.drawText(QRectF(int(x * x_gap), int((y) * y_gap),300,300), str(star_planets))
            painter.setPen(QPen())
    def paintEvent(self, event):
        self.createUI()
class SapthaShalaka(QWidget):
    def __init__(self,planet_positions=[],planets_in_retrograde=[],label_font_size=6):
        super().__init__()
        self.x_dim = 9; self.y_dim = 9; self.x_gap = 50; self.y_gap=50
        self.setGeometry(100, 100, 500, 500)
        self.setWindowTitle(utils.resource_strings['saptha_shalaka_str']+' '+utils.resource_strings['chakra_str'])
        self.lines = [((2,1),(2,9)),((3,1),(3,9)),((4,1),(4,9)),
                      ((5,1),(5,9)),((6,1),(6,9)),((7,1),(7,9)),
                      ((8,1),(8,9)),((1,2),(9,2)),((1,3),(9,3)),((1,4),(9,4)),
                      ((1,5),(9,5)),((1,6),(9,6)),((1,7),(9,7)),((1,8),(9,8))
                      ]
        self.rasi_labels = {
            1: (7, 1), 2: (8, 1), 3: (9, 2), 4: (9, 3), 5: (9, 4), 6: (9, 5), 7: (9, 6),
            8: (9, 7), 9: (9, 8), 10: (8, 9), 11: (7, 9), 12: (6, 9), 13: (5, 9), 14: (4, 9),
            15: (3, 9), 16: (2, 9), 17: (1, 8), 18: (1, 7), 19: (1, 6), 20: (1, 5),
            21: (1, 4), 22: (1, 3), 23: (1,2), 24: (2, 1), 25: (3, 1), 26: (4, 1),
            27: (5, 1), 28: (6, 1)
        }
        self._label_font_size = label_font_size
        self._base_star = 1
        self._update_with_star_labels()
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
    def _update_with_star_labels(self):
        _star_list = utils.NAKSHATRA_SHORT_LIST
        _abhijit_list = [_star_list[i] for i in const.abhijit_order_of_stars]
        _star_labels = {}
        for k,(x,y) in self.rasi_labels.items():
            nak = ((self._base_star-1)+(k-1))%28
            #self.rasi_labels[k] = (x,y,_abhijit_list[nak])
            _star_labels[nak+1] = (x,y,_abhijit_list[nak])
        self.rasi_labels = _star_labels.copy()
    def _update_with_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            if p_star > 21: p_star += 1
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            x,y,pl = self.rasi_labels[p_star]
            self.rasi_labels[p_star] = [x,y,pl+'\n'+pstr+rstr]        
    def setData(self,planet_positions,planets_in_retrograde=[],label_font_size=None):
        if label_font_size != None: self._label_font_size = label_font_size
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._update_with_planet_labels()
        self.createUI()
    def createUI(self):
        painter = QPainter(self)
        painter.setPen(QPen(Qt.GlobalColor.black, 2, Qt.PenStyle.SolidLine))
        x_gap = self.x_gap; y_gap = self.y_gap
        # Draw lines
        for line in self.lines:
            start, end = line
            painter.drawLine(start[0] * x_gap, (start[1]) * y_gap, end[0] * x_gap, (end[1]) * y_gap)

        # Draw text
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold); painter.setFont(font)
        for _, (x, y,star_planets) in self.rasi_labels.items():
            color = self._planet_color if '\n' in star_planets else self._star_color
            painter.setPen(QColor(color))
            painter.drawText(QRectF(int(x * x_gap), int((y) * y_gap),300,300), str(star_planets))
            painter.setPen(QPen())
    def paintEvent(self, event):
        self.createUI()
RahuKalanala = SapthaShalaka
class ChandraKalanala(QWidget):
    def __init__(self,base_star=18,planet_positions=[],planets_in_retrograde=[],label_font_size=6):
        super().__init__()
        self.x_dim = 8; self.y_dim = 9; self.x_gap = 60; self.y_gap=55
        self.setGeometry(0, 0, 500, 500)
        self.setWindowTitle(utils.resource_strings['chandra_kalanala_str']+' '+utils.resource_strings['chakra_str'])
        self.lines = [((4,8),(4,7)),((4,7),(6,7)),((6,7),(6,8)),((5,8),(5,2)),((4,2),(4,3)),((4,3),(6,3)),((6,3),(6,2)),
                      ((2,4),(3,4)),((3,4),(3,6)),((3,6),(2,6)),((2,5),(8,5)),((8,4),(7,4)),((7,4),(7,6)),((7,6),(8,6))]
        self.rasi_labels = {
            1: (5,1), 2: (4,1), 3: (4,3), 6: (3,4), 7: (2,4),
            8: (2,5), 9: (2,6), 10: (3, 6), 13: (4,7), 14: (4,8),
            15: (5,8), 16: (6,8), 17: (6,7), 20: (7,6),
            21: (8,6), 22: (8,5), 23: (8,4), 24: (7,4),
            27: (6,3), 28: (6,2), 4:(4,4),5:(4,5),11:(4,6),12:(5,6),18:(6,6),19:(6,5),25:(6,4),26:(5,4),
        }
        self._label_font_size = label_font_size
        self._base_star = base_star
        self._update_with_star_labels()
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
    def _update_with_star_labels(self):
        _star_list = utils.NAKSHATRA_SHORT_LIST
        _abhijit_list = [_star_list[i] for i in const.abhijit_order_of_stars]
        _star_labels = {}
        for k,(x,y) in self.rasi_labels.items():
            nak = ((self._base_star-1)+(k-1))%28
            #self.rasi_labels[k] = (x,y,_abhijit_list[nak])
            _star_labels[nak+1] = (x,y,_abhijit_list[nak])
        self.rasi_labels = _star_labels.copy()
    def _update_with_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            if p_star > 21: p_star += 1
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            x,y,pl = self.rasi_labels[p_star]
            self.rasi_labels[p_star] = [x,y,pl+'\n'+pstr+rstr]        
    def setData(self,planet_positions,planets_in_retrograde=[],base_star=None,label_font_size=None):
        if base_star != None: self._base_star = base_star
        if label_font_size != None: self._label_font_size = label_font_size
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._update_with_planet_labels()
        self.createUI()
    def createUI(self):
        painter = QPainter(self)
        painter.setPen(QPen(Qt.GlobalColor.black, 2, Qt.PenStyle.SolidLine))
        x_gap = self.x_gap; y_gap = self.y_gap
        # draw cicrle
        p1 = (1,1); p2 = (2,1)
        rad = 1.5*x_gap
        center = QPointF(5*x_gap,5*y_gap)
        painter.drawEllipse(center, rad, rad)
        # Draw lines
        for line in self.lines:
            start, end = line
            painter.drawLine(start[0] * x_gap, (start[1]) * y_gap, end[0] * x_gap, (end[1]) * y_gap)

        # Draw text
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold); painter.setFont(font)
        for _, (x, y,star_planets) in self.rasi_labels.items():
            color = self._planet_color if '\n' in star_planets else self._star_color
            painter.setPen(QColor(color))
            painter.drawText(QRectF(int(x * x_gap), int((y) * y_gap),300,300), str(star_planets))
            painter.setPen(QPen())
    def paintEvent(self, event):
        self.createUI()
class Tripataki(QWidget):
    def __init__(self,planet_positions=[],planets_in_retrograde=[],label_font_size=6):
        super().__init__()
        self.x_dim = 5; self.y_dim = 5; self.x_gap = 80; self.y_gap=80
        self.setGeometry(0, 0, 500, 500)
        self.setWindowTitle(utils.resource_strings['tripataki_str']+' '+utils.resource_strings['chakra_str'])
        self.lines = {(2,5):[(1,4),(2,1),(5,2)],(3,5):[(1,3),(3,1),(5,3)],(4,5):[(1,2),(4,1),(5,4)],
                      (2,1):[(1,2),(5,4)],(3,1):[(1,3),(5,3)],(4,1):[(1,4),(5,2)],
                      (1,2):[(5,2)],(1,3):[(5,3)],(1,4):[(5,4)]
                       }
        self.rasi_labels = [(1,3),(1,4),(2,5),(3,5),(4,5),(5,4),(5,3),(5,2),(4,1),(3,1),(2,1),(1,2)]
        self._label_font_size = label_font_size
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
    def _update_with_planet_labels(self):
        self.rasi_labels = [(x,y,utils.RAASI_SHORT_LIST[h]) for h,(x,y) in enumerate(self.rasi_labels)]
        for p,(h,_) in self._planet_positions:
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            x,y,pl = self.rasi_labels[h]
            self.rasi_labels[h] = [x,y,pl+'\n'+pstr+rstr]        
    def setData(self,planet_positions,planets_in_retrograde=[],base_star=None,label_font_size=None):
        if base_star != None: self._base_star = base_star
        if label_font_size != None: self._label_font_size = label_font_size
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._update_with_planet_labels()
        self.createUI()
    def createUI(self):
        painter = QPainter(self)
        painter.setPen(QPen(Qt.GlobalColor.black, 2, Qt.PenStyle.SolidLine))
        x_gap = self.x_gap; y_gap = self.y_gap
        # Draw lines
        for start,ends in self.lines.items():
            for end in ends:
                painter.drawLine(start[0] * x_gap, (start[1]) * y_gap, end[0] * x_gap, (end[1]) * y_gap)

        # Draw text
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold); painter.setFont(font)
        for x, y,rasi_planets in self.rasi_labels:
            color = self._planet_color if '\n' in rasi_planets else self._rasi_color
            painter.setPen(QColor(color))
            painter.drawText(QRectF(int(x * x_gap), int((y) * y_gap),300,300), str(rasi_planets))
            painter.setPen(QPen())
    def paintEvent(self, event):
        self.createUI()
class SuryaKalanala(QWidget):
    def __init__(self,base_star=18,planet_positions=[],planets_in_retrograde=[],label_font_size=6):
        super().__init__()
        self.x_dim = 15; self.y_dim = 15; self.x_gap = 30; self.y_gap=30
        self.setGeometry(100, 100, 500, 500)
        self.setWindowTitle(utils.resource_strings['surya_kalanala_str']+' '+utils.resource_strings['chakra_str'])
        self.lines = [((1,6),(15,6)),((1,8),(15,8)),((1,10),(15,10)),
                      ((5,1),(5,15)),((8,1),(8,15)),((11,1),(11,15)),
                      ((4,6),(2,3)),((5,6),(2,2)),((5,5),(3,2)),((5,4),(4,2)),
                      ((12,6),(14,3)),((11,6),(14,2)),((11,5),(13,2)),((11,4),(12,2)),
                      ((5,12),(3,14)),((3,10),(2,12)),((11,12),(13,14)),((13,10),(14,12)),
                      ((5,4),(6,2)),((8,4),(7,2)),((8,4),(9,2)),((11,4),(12,2)),((11,4),(10,2))
                      ]
        self.rasi_labels = {
            1: (8, 15), 2: (5, 15), 3: (3, 15), 4: (1, 14), 5: (1, 10), 6: (1, 8), 7: (1, 6),
            8: (1, 4), 9: (1, 1), 10: (3, 1), 11: (4, 1), 12: (5, 1), 13: (6, 1), 14: (7, 1),
            15: (8, 1), 16: (9, 1), 17: (10, 1), 18: (11, 1), 19: (12, 1), 20: (13, 1),
            21: (15, 1), 22: (15, 4), 23: (15,6), 24: (15, 8), 25: (15, 10), 26: (15, 14),
            27: (13, 15), 28: (11, 15)
        }
        self._label_font_size = label_font_size
        self._base_star = base_star
        self._update_with_star_labels()
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
    def _update_with_star_labels(self):
        _star_list = utils.NAKSHATRA_SHORT_LIST
        _abhijit_list = [_star_list[i] for i in const.abhijit_order_of_stars]
        _star_labels = {}
        for k,(x,y) in self.rasi_labels.items():
            nak = ((self._base_star-1)+(k-1))%28
            #self.rasi_labels[k] = (x,y,_abhijit_list[nak])
            _star_labels[nak+1] = (x,y,_abhijit_list[nak])
        self.rasi_labels = _star_labels.copy()
    def _update_with_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            if p_star > 21: p_star += 1
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            x,y,pl = self.rasi_labels[p_star]
            self.rasi_labels[p_star] = [x,y,pl+'\n'+pstr+rstr]        
    def setData(self,planet_positions,planets_in_retrograde=[],base_star=None,label_font_size=None):
        if base_star != None: self._base_star = base_star
        if label_font_size != None: self._label_font_size = label_font_size
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._update_with_planet_labels()
        self.createUI()
    def createUI(self):
        painter = QPainter(self)
        painter.setPen(QPen(Qt.GlobalColor.black, 2, Qt.PenStyle.SolidLine))
        x_gap = self.x_gap; y_gap = self.y_gap
        # Draw lines
        for line in self.lines:
            start, end = line
            painter.drawLine(start[0] * x_gap, (start[1]) * y_gap, end[0] * x_gap, (end[1]) * y_gap)

        # Draw text
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold); painter.setFont(font)
        for _, (x, y,star_planets) in self.rasi_labels.items():
            color = self._planet_color if '\n' in star_planets else self._star_color
            painter.setPen(QColor(color))
            painter.drawText(QRectF(int(x * x_gap), int((y) * y_gap),300,300), str(star_planets))
            painter.setPen(QPen())
    def paintEvent(self, event):
        self.createUI()
class Shoola(QWidget):
    def __init__(self,base_star=18,planet_positions=[],planets_in_retrograde=[],label_font_size=6):
        super().__init__()
        self.x_dim = 11; self.y_dim = 11; self.x_gap = 30; self.y_gap=40
        self.setGeometry(100, 100, 400, 600)
        self.setWindowTitle(utils.resource_strings['shoola_str']+' '+utils.resource_strings['chakra_str'])
        self.lines = [
            ((5, 1), (5, 10)),
            ((2, 7), (8, 7)),
            ((2, 7), (2, 10)),
            ((8, 7), (8, 10))
        ]
        self.rasi_labels = {
            1: (5, 1), 2: (4, 2), 3: (4, 3), 4: (4, 4), 5: (4, 5), 6: (4, 6), 7: (4, 7),
            8: (3, 10), 9: (2, 11), 10: (1, 10), 11: (1, 9), 12: (1, 8), 13: (2, 7),
            14: (4, 10), 15: (5, 11), 16: (6, 10), 17: (6, 9), 18: (6, 8.5), 19: (6, 8),
            20: (7, 10), 21: (8, 11), 22: (9, 10), 23: (9, 9), 24: (9, 8), 25: (8, 7),
            26: (6, 4), 27: (6, 3), 28: (6, 2)
        }
        self._label_font_size = label_font_size
        self._base_star = base_star
        self._update_with_star_labels()
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
    def _update_with_star_labels(self):
        _star_list = utils.NAKSHATRA_SHORT_LIST
        _abhijit_list = [_star_list[i] for i in const.abhijit_order_of_stars]
        _star_labels = {}
        for k,(x,y) in self.rasi_labels.items():
            nak = ((self._base_star-1)+(k-1))%28
            #self.rasi_labels[k] = (x,y,_abhijit_list[nak])
            _star_labels[nak+1] = (x,y,_abhijit_list[nak])
        self.rasi_labels = _star_labels.copy()
    def _update_with_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            if p_star > 21: p_star += 1
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            x,y,pl = self.rasi_labels[p_star]
            self.rasi_labels[p_star] = [x,y,pl+'\n'+pstr+rstr]        
    def setData(self,planet_positions,planets_in_retrograde=[],base_star=None,label_font_size=None):
        if base_star != None: self._base_star = base_star
        if label_font_size != None: self._label_font_size = label_font_size
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._update_with_planet_labels()
        self.createUI()
    def createUI(self):
        painter = QPainter(self)
        painter.setPen(QPen(Qt.GlobalColor.black, 2, Qt.PenStyle.SolidLine))
        x_gap = self.x_gap; y_gap = self.y_gap
        # Draw lines
        for line in self.lines:
            start, end = line
            painter.drawLine(start[0] * x_gap, (self.y_dim+1 - start[1]) * y_gap, end[0] * x_gap, (self.y_dim+1 - end[1]) * y_gap)

        # Draw text
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold); painter.setFont(font)
        for _, (x, y,star_planets) in self.rasi_labels.items():
            color = self._planet_color if '\n' in star_planets else self._star_color
            painter.setPen(QColor(color))
            painter.drawText(QRectF(int(x * x_gap), int((self.y_dim+1 - y) * y_gap),300,300), str(star_planets))
            painter.setPen(QPen())
    def paintEvent(self, event):
        self.createUI()
class Sarvatobadra(QWidget):
    def __init__(self, planet_positions=[],planets_in_retrograde=[]):
        QWidget.__init__(self)
        self._grid_label_values = [
            ['ii',  23, 24, 25, 26, 27, 1, 2, 'a'],
            [22, 'rii', 'g', 's', 'd', 'ch', 'l', 'u', 3],
            [28, 'kh', 'ai', 11, 12, 1, 'lu', 'a', 4],
            [21, 'j', 10, 'ah', 'Rikita<br>Fri', 'o', 2, 'v', 5],
            [20, 'bh', 9, 'Jaya<br>Thu', 'Pooma<br>Sat', 'Nanda<br>Sun<br>Tue', 3, 'k', 6],
            [19, 'y', 8, 'am', 'Bhadra<br>Mon<br>Wed', 'au', 4, 'h', 7],
            [18, 'n', 'e', 7, 6, 5, 'luu', 'd', 8],
            [17, 'ri', 't', 'r', 'p', 't~', 'm', 'uu', 9],
            ['i', 16, 15, 14, 13, 12, 11, 10, 'aa']
        ]
        self._star_indices = {self._grid_label_values[i][j]:(i,j) for i in range(9) for j in range(9) if ((i==8) or (j==8) or (i==0) or (j==0)) and isinstance(self._grid_label_values[i][j],int)}
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions = planet_positions
        self._planets_in_retrograde=planets_in_retrograde
        self.createUI()
    def createUI(self):
        self.layout = QVBoxLayout()
        self.gridLayout = QGridLayout()
        self._grid_labels = [[QLabel('') for j in range(9)] for i in range(9)]
        for i in range(9):
            for j in range(9):
                #self._grid_labels[i][j].setText(self._grid_label_values[i][j])
                self._grid_labels[i][j].setStyleSheet("border: 1px solid black;")  # Optional: Add a border to each cell
                self.gridLayout.addWidget(self._grid_labels[i][j], i, j)
        self.layout.addLayout(self.gridLayout)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setContentsMargins(0,0,0,0)
        self.setLayout(self.layout)
        self._update_labels_with_stars_and_rasis()
        if len(self._planet_positions) > 0:
            self._update_with_planet_labels()
    def _update_with_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            r,c = self._star_indices[p_star]
            cell_value = self._grid_labels[r][c].text()+'<br>'+color_label(pstr+rstr,self._planet_color)
            self._grid_labels[r][c].setText(cell_value)
        self.update()
    def setData(self,planet_positions=[],planets_in_retrograde=[],label_font_size=8):
        if len(planet_positions) > 0:
            self._planet_positions = planet_positions
            self._update_with_planet_labels()
    def _update_labels_with_stars_and_rasis(self):
        for i in range(9):
            for j in range(9):
                cell_value = self._grid_label_values[i][j]
                border_cell = ((i==8) or (j==8)) or (i==0) or (j==0)
                if border_cell and isinstance(cell_value,int):
                    cell_value = color_label(utils.NAKSHATRA_SHORT_LIST[cell_value-1],self._star_color)
                    self._grid_labels[i][j].setText(cell_value)
                    self._grid_labels[i][j].setAlignment(Qt.AlignmentFlag.AlignTop)
                elif not border_cell and isinstance(cell_value,int):
                    cell_value = color_label(utils.RAASI_SHORT_LIST[cell_value-1],self._rasi_color)
                    self._grid_labels[i][j].setText(cell_value)
                    self._grid_labels[i][j].setAlignment(Qt.AlignmentFlag.AlignCenter)
                else:
                    cell_value = color_label(cell_value,self._text_color)
                    self._grid_labels[i][j].setText(cell_value)
                    self._grid_labels[i][j].setAlignment(Qt.AlignmentFlag.AlignCenter)
class KaalaChakra(QWidget):
    def __init__(self, planet_positions=[],planets_in_retrograde=[],base_star=None,
                 title_font_size=8,label_font_size=8):
        """
            If birth chart Rasi - sun_star = star of Sun's longitude
            For all other chars - use star of Lagna longitude
        """
        QWidget.__init__(self)
        self._rasi_color = 'Red'; self._planet_color = 'Brown'; self._text_color='Green';
        self._star_color = 'Blue'
        self._planet_positions=[]
        self._label_font_size = label_font_size
        if len(planet_positions)>0:
            self.createUI()
    def get_kaala_chakra_star_labels(self):
        _star_list = utils.NAKSHATRA_SHORT_LIST
        _abhijit_list = [_star_list[i] for i in const.abhijit_order_of_stars]
        self.stars_inner = [((self._base_star-1)+(ele-1))%28 for ele in self.stars_inner]
        self.labels_inner = [_abhijit_list[i] for i in self.stars_inner]
        self.stars_outer_divisions = [[((self._base_star-1)+(ele-1))%28  for ele in row] for row in self.stars_outer_divisions]
        self.labels_outer_divisions = [[_abhijit_list[i]  for i in row] for row in self.stars_outer_divisions]
    def createUI(self):
        qp = QPainter(self)
        qp.begin(self)
        try:
            self.drawCircle(qp)
        except Exception as e:
            print(f"Error in drawCircle: {e}")
        qp.end()
    def setData(self,planet_positions=[],planets_in_retrograde=[],base_star=18,
                 title_font_size=8,label_font_size=8):
        self._label_font_size = label_font_size
        self._base_star = base_star
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self.labels_custom = [utils.resource_strings[r] for r in ['kaala_chakra_southeast_str','kaala_chakra_east_str','kaala_chakra_northeast_str',
                                'kaala_chakra_north_str','kaala_chakra_northwest_str','kaala_chakra_west_str',
                                'kaala_chakra_southwest_str','kaala_chakra_south_str']]
        self.stars_inner = [4, 25, 18, 11]
        self.stars_outer_divisions = [[5, 6, 7],[3, 2, 1],[26, 27, 28],[24, 23, 22],[19, 20, 21],
                                      [17, 16, 15],[12, 13, 14],[10, 9, 8]]
        self.get_kaala_chakra_star_labels()
        self.add_planet_labels()
        self.inner_angles = [45, 135, 225, 315]
        self.outer_angles = [45, 90, 135, 180, 225, 270, 315, 360]
        self.createUI()
    def add_planet_labels(self):
        for p,(h,long) in self._planet_positions:
            p_long = h*30+long
            from jhora.panchanga import drik
            nak = drik.nakshatra_pada(p_long)
            p_star = nak[0]
            if p_star > 21: p_star += 1
            rstr = const._retrogade_symbol if p in self._planets_in_retrograde else ''
            pstr = utils.resource_strings['ascendant_short_str'] if p==const._ascendant_symbol else utils.PLANET_SHORT_NAMES[p]
            if p_star-1 in self.stars_inner:
                self.labels_inner[self.stars_inner.index(p_star-1)] += '\n'+pstr+rstr
            else:
                r,c = get_2d_list_index(self.stars_outer_divisions,p_star-1)
                self.labels_outer_divisions[r][c] += '\n'+pstr+rstr
    def paintEvent(self, event):
        if len(self._planet_positions) > 0:
            self.createUI()
    def drawCircle(self, qp):
        try:
            # Set up pen and font
            pen = QPen(Qt.GlobalColor.black, 2)
            qp.setPen(pen)
            font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold)
            qp.setFont(font)

            # Draw circle
            center = QPointF(self.width() / 2, self.height() / 2)
            radius = min(self.width(), self.height()) / 8
            qp.drawEllipse(center, radius, radius)

            # Draw labels inside the circle
            angles_inner = self.inner_angles
            for angle, label in zip(angles_inner, self.labels_inner):
                rad = math.radians(angle)
                x = int(center.x() + radius * 0.5 * math.cos(rad))
                y = int(center.y() - radius * 0.5 * math.sin(rad))
                color = self._planet_color if '\n' in label else self._star_color
                qp.setPen(QColor(color))
                qp.drawText(QRectF(x,y, 300, 300),Qt.AlignmentFlag.AlignLeft, label)
                qp.setPen(QPen())
            # Draw lines and labels outside the circle
            angles_outer = self.outer_angles
            for i, angle in enumerate(angles_outer):
                rad = math.radians(angle)
                x_start = int(center.x() + radius * math.cos(rad))
                y_start = int(center.y() - radius * math.sin(rad))
                x_outer = int(center.x() + (radius + 200) * math.cos(rad))
                y_outer = int(center.y() - (radius + 200) * math.sin(rad))
                qp.drawLine(QPointF(x_start, y_start), QPointF(x_outer, y_outer))

                # Draw labels at each division of the lines
                for j in range(1, 4):
                    x_div = int(center.x() + (radius + 200 / 4 * j) * math.cos(rad))
                    y_div = int(center.y() - (radius + 200 / 4 * j) * math.sin(rad))
                    color = self._planet_color if '\n' in self.labels_outer_divisions[i][j-1] else self._star_color
                    qp.setPen(QColor(color))
                    qp.drawText(QRectF(x_div,y_div, 300, 300),Qt.AlignmentFlag.AlignLeft, self.labels_outer_divisions[i][j-1])
                    qp.setPen(QPen())
                # Draw the final custom label at the end of each line
                x_label = int(center.x() + (radius + 220) * math.cos(rad))
                y_label = int(center.y() - (radius + 220) * math.sin(rad))
                if angle == 180 or angle==360:
                    y_label -= 20
                elif angle == 90:
                    x_label += 40; y_label += 60
                elif angle == 270:
                    x_label += 40; y_label -= 60
                qp.setPen(QColor(self._text_color))
                qp.drawText(QRectF(x_label,y_label, 300, 300),Qt.AlignmentFlag.AlignLeft, self.labels_custom[i])
                qp.setPen(QPen())
        except Exception as e:
            print(f"Error in drawCircle: {e}")

    def closeEvent(self, event):
        event.accept()
            
class KotaChakra(QWidget):
    def __init__(self, planet_positions=[], birth_star=15,birth_star_padha=1,planets_in_retrograde=[],
                 title_font_size=12,label_font_size=12,
                 square_color_rgbs=[(255, 128, 128),(128, 255, 128), (128, 128, 255), (255, 255, 128)],
                 square_captions=[]):
        QWidget.__init__(self)
        self._star_list = utils.NAKSHATRA_SHORT_LIST
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._birth_star = birth_star
        self._birth_star_padha = birth_star_padha
        self.labels = self.get_planet_star_labels(planet_positions, birth_star,planets_in_retrograde)
        self._title_font_size = title_font_size
        self._label_font_size = label_font_size
        self._square_color_rgbs = square_color_rgbs
        if len(square_captions)==0:
            self._square_captions = [utils.resource_strings[r] for r in ['baahya_str','praakaara_str','durgantara_str','sthamba_str']]
        else:
            self._square_captions = square_captions
        self._space_between_squares = 65
        self.setGeometry(0,0,550,550)
        self.setWindowTitle(utils.resource_strings['kota_str']+' '+utils.resource_strings['chakra_str'])
        if len(self._planet_positions)>0:
            moon_house = self._planet_positions[2][1][0]
            from jhora.horoscope.chart import house
            self._kota_lord = utils.PLANET_NAMES[house.house_owner_from_planet_positions(planet_positions, moon_house)]
            self._kota_paala = utils.PLANET_NAMES[const.kota_paala_lord_for_star_paadha[birth_star-1][birth_star_padha-1]]
            #print('kota lord',self._kota_lord,'kota paala',self._kota_paala)
    def createUI(self):
        self._star_list = utils.NAKSHATRA_SHORT_LIST
        painter = QPainter(self)
        rect = self.rect().adjusted(0, 0, -1, -1)
        colors = [QColor(x[0],x[1],x[2]) for x in self._square_color_rgbs ]
        # Draw 4 concentric squares
        num_squares = 4
        for i in range(num_squares):
            inset = i * self._space_between_squares  # Space between squares
            square_rect = rect.adjusted(inset, inset, -inset, -inset)
            painter.fillRect(square_rect, colors[i])
            self.drawLabels(painter, square_rect, i, num_squares)
            self.drawCaptions(painter, square_rect, i, num_squares)
    def setData(self,planet_positions=[], birth_star=15,birth_star_padha=1,planets_in_retrograde=[],
                 title_font_size=12,label_font_size=8,
                 square_color_rgbs=[(255, 128, 128),(128, 255, 128), (128, 128, 255), (255, 255, 128)],
                 square_captions=[]):
        self._planet_positions = planet_positions
        self._planets_in_retrograde = planets_in_retrograde
        self._birth_star = birth_star
        self.labels = self.get_planet_star_labels(planet_positions, birth_star,planets_in_retrograde)
        self._title_font_size = title_font_size
        self._label_font_size = label_font_size
        self._square_color_rgbs = square_color_rgbs
        if len(square_captions)==0:
            self._square_captions = [utils.resource_strings[r] for r in ['baahya_str','praakaara_str','durgantara_str','sthamba_str']]
        else:
            self._square_captions = square_captions
        moon_house = self._planet_positions[2][1][0]
        from jhora.horoscope.chart import house
        self._kota_lord = utils.PLANET_NAMES[house.house_owner_from_planet_positions(planet_positions, moon_house)]
        self._kota_paala = utils.PLANET_NAMES[const.kota_paala_lord_for_star_paadha[birth_star-1][birth_star_padha-1]]
        #print('kota lord',self._kota_lord,'kota paala',self._kota_paala)
        self.createUI()
    def paintEvent(self, event):
        self.createUI()
    def drawLabels(self, painter, rect, index, num_squares):
        painter.setPen(Qt.GlobalColor.black)
        # Retrieve labels from the list
        square_labels = self.labels[index]
        xadj = int(0.02*rect.width()); yadj = int(0.03*rect.height())
        if index == num_squares - 1:  # Innermost square
            label_positions = [
                (QPoint(rect.center().x()+xadj, rect.top()+yadj), square_labels[0]), #Inner Top Middle
                (QPoint(rect.right()-xadj, rect.center().y()), square_labels[1]), # Inner Right
                (QPoint(rect.center().x()+xadj, rect.bottom()-3*yadj), square_labels[2]), # Inner Bottom
                (QPoint(rect.left()+3*xadj, rect.center().y()), square_labels[3]) # Inner Left
            ]
        else:
            label_positions = [
                (rect.topLeft()+QPoint(4*xadj,yadj), square_labels[0]), # Top Left
                (QPoint(rect.center().x()+xadj, rect.top()+yadj), square_labels[1]), # Top Middle
                (rect.topRight()+QPoint(-xadj,yadj), square_labels[2]), # Top Right
                (QPoint(rect.right()-xadj, rect.center().y()), square_labels[3]), # Center Right
                (rect.bottomRight()+QPoint(-xadj,-3*yadj), square_labels[4]), # Bottom Right
                (QPoint(rect.center().x()+xadj, rect.bottom()-3*yadj), square_labels[5]), # Bottom Middle
                (rect.bottomLeft()+QPoint(4*xadj,-3*yadj), square_labels[6]), # Bottom Left
                (QPoint(rect.left()+4*xadj, rect.center().y()), square_labels[7]) # Center Left
            ]
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold)
        painter.setFont(font)
        for pos, text in label_positions:
            painter.drawText(QRectF(pos.x() - 30, pos.y() - 10, 60, 200), Qt.AlignmentFlag.AlignLeft, text)
        self.update()
    def drawCaptions(self, painter, rect, index, num_squares):
        painter.setPen(Qt.GlobalColor.black)
        _caption = self._square_captions[index]
        xadj = 10; yadj = 15
        if index == num_squares - 1:  # Innermost square
            pos = (QPoint(rect.bottomRight().x()-xadj, rect.bottomRight().y()-int(yadj/2)))
        else:
            pos = (QPoint(rect.bottomRight().x()-10*xadj, rect.bottom()-int(yadj/2)))
        font = QFont(); font.setPointSize(self._label_font_size); font.setWeight(QFont.Weight.Bold)
        painter.setFont(font)
        painter.drawText(QRectF(pos.x() - 30, pos.y() - 10, 300, 300), Qt.AlignmentFlag.AlignLeft, _caption)
    def get_kota_chakra_star_list(self,birth_star=15):
        star_list_1 = [self._star_list[i] for i in const.abhijit_order_of_stars]
        return [[star_list_1[((birth_star-1)+(ele-1))%28]  for ele in row] for row in const.kota_chakra_star_placement_from_birth_star]
         
    def get_labels_from_birth_star(self,birth_star=15):
        star_list_1 = [self._star_list[i] for i in const.abhijit_order_of_stars]
        #star_list_1 = [i for i in const.abhijit_order_of_stars]
        labels = []
        for row in const.kota_chakra_star_placement_from_birth_star:
            lbl = []
            for ele in row:
                star = star_list_1[((birth_star-1)+(ele-1))%28]
                lbl.append(str(star))
            labels.append(lbl)
        return labels
    def get_planet_star_labels(self,planet_positions,birth_star=15,planets_in_retrograde=[]):
        from jhora.panchanga import drik
        kcs_list = self.get_kota_chakra_star_list(birth_star)
        labels = self.get_labels_from_birth_star(birth_star)
        for planet, (sign,long) in planet_positions:
            rStr = const._retrogade_symbol if planet in planets_in_retrograde else ''
            p_long = sign*30+long
            nak,_,_ = drik.nakshatra_pada(p_long)
            p_i,p_j = get_2d_list_index(kcs_list,self._star_list[nak-1])
            if  planet==const._ascendant_symbol:
                labels[p_i][p_j] += '\n'+utils.resource_strings['ascendant_short_str']
            else:
                labels[p_i][p_j] += '\n' + utils.PLANET_SHORT_NAMES[planet]+rStr
        return labels
    def resizeEvent(self, event):
        side = min(self.width(), self.height())
        self.setFixedSize(side, side)

if __name__ == "__main__":
    utils.set_language('ta')
    def except_hook(cls, exception, traceback):
        print('exception called')
        sys.__excepthook__(cls, exception, traceback)
    sys.excepthook = except_hook
    App = QApplication(sys.argv)
    planet_positions = [['L', (9, 22.44575902738461)], [0, (7, 21.565282200247736)], [1, (6, 6.959489445251251)], 
                        [2, (4, 25.539747317445176)], [3, (8, 9.936449034377262)], [4, (8, 25.82805158497206)], 
                        [5, (6, 23.71713310535756)], [6, (11, 6.807276353388318)], [7, (5, 10.553787374448945)], 
                        [8, (11, 10.553787374448916)],[9,(9,8.2)],[10,(9,2.2)],[11,(7,9.7)]]
    chart = SapthaNaadi()
    chart.setData(planet_positions, planets_in_retrograde=[3,4])
    #chart = PanchaShalaka()#RahuKalanala()
    #chart.setData(planet_positions=planet_positions,planets_in_retrograde=[3,4])
    #chart = SapthaShalaka()#RahuKalanala()
    #chart.setData(planet_positions=planet_positions,planets_in_retrograde=[3,4])
    #chart = ChandraKalanala(base_star=18)
    #chart.setData(planet_positions=planet_positions,planets_in_retrograde=[3,4])
    #chart = Tripataki()
    #chart.setData(planet_positions=planet_positions,planets_in_retrograde=[3,4])
    #chart = SuryaKalanala(base_star=18)
    #chart.setData(planet_positions=planet_positions,planets_in_retrograde=[3,4])
    #chart = Shoola(base_star=18)
    #chart.setData(planet_positions=planet_positions,planets_in_retrograde=[3,4])
    #chart = KotaChakra()
    #chart.setData(planet_positions=planet_positions,birth_star=15,planets_in_retrograde=[3,5],label_font_size=6)
    #chart = KaalaChakra()
    #chart.setData(planet_positions=planet_positions,base_star=18,planets_in_retrograde=[3,5],label_font_size=6)
    #chart = Sarvatobadra()
    #chart.setData(planet_positions, planets_in_retrograde=[3,4])
    chart.show()
    sys.exit(App.exec())



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
from jhora.horoscope.chart import charts, house
""" method = 2 KN Rao Method - Working. Method 1=> Sanjay Rath - yet to be implemented """
sidereal_year = const.sidereal_year
dhasa_order = {0:([0,3,6,9, 2,5,8,11, 1,4,7,10],[0,9,6,3, 2,11,8,5, 1,10,7,4]),
               3:([3,6,9,0, 2,5,8,11, 1,4,7,10],[3,0,9,6, 2,11,8,5, 1,10,7,4]),
               6:([6,9,0,3, 2,5,8,11, 1,4,7,10],[6,3,0,9, 2,11,8,5, 1,10,7,4]),
               9:([9,0,3,6, 2,5,8,11, 1,4,7,10],[9,6,3,0, 2,11,8,5, 1,10,7,4]),
               2:([2,5,8,11, 1,4,7,10, 0,3,6,9],[2,11,8,5, 1,10,7,4,0,9,6,3]),
               5:([5,8,11,2, 1,4,7,10,0,3,6,9],  [5,2,11,8, 1,10,7,4,0,9,6,3]),
               8:([8,11,2,5, 1,4,7,10,0,3,6,9 ],[8,5,2,11, 1,10,7,4,0,9,6,3]),
               11:([11,2,5,8, 1,4,7,10, 0,3,6,9],[11,8,5,2, 1,10,7,4,0,9,6,3]),
               1:([1,4,7,10, 0,3,6,9, 2,5,8,11],[1,10,7,4, 0,9,6,3, 2,11,8,5]),
               4:([4,7,10,1, 0,3,6,9, 2,5,8,11],[4,1,10,7, 0,9,6,3, 2,11,8,5]),
               7:([7,10,1,4, 0,3,6,9, 2,5,8,11],[7,4,1,10, 0,9,6,3, 2,11,8,5]),
               10:([10,1,4,7, 0,3,6,9, 2,5,8,11],[10,7,4,1, 0,9,6,3, 2,11,8,5])
               }
               
def _dhasa_duration_kn_rao(planet_positions,sign):
    p_to_h = utils.get_planet_house_dictionary_from_planet_positions(planet_positions)
    h_to_p = utils.get_house_to_planet_dict_from_planet_to_house_dict(p_to_h)
    lord_of_sign = house.house_owner_from_planet_positions(planet_positions, sign)
    house_of_lord = p_to_h[lord_of_sign]
    #print('dhasa_lord',sign,'lord_of_sign',lord_of_sign,'house_of_lord',house_of_lord,'strength',const.house_strengths_of_planets[lord_of_sign][house_of_lord])
    dhasa_period = 0
    #print('start woth dhasa years 0 - sign lord of sign house of lord dhasa years',sign,lord_of_sign,house_of_lord,dhasa_period)
    """ The length of a dasa is determined by the position of the lord of dasa rasi with respect to dasa rasi."""
    if sign in const.even_footed_signs: # count back from sign to house_of_lord
        dhasa_period = (sign-house_of_lord+1)%12
            #print('house_of_lord',house_of_lord,'> sign',sign,'dhasa_period',dhasa_period)
    else:
        dhasa_period = (house_of_lord-sign+1)%12
    if dhasa_period <=0 or const.house_strengths_of_planets[lord_of_sign][house_of_lord] == const._OWNER_RULER:# or \
            #house_of_lord==(sign+11)%12:
        dhasa_period = 12
    if house_of_lord==(sign+6)%12:
        dhasa_period = 10
    return dhasa_period
def _dhasa_duration(lord):
    if lord in const.movable_signs:
        return 7
    elif lord in const.fixed_signs:
        return 8
    else:
        return 9
def get_dhasa_antardhasa(dob,tob,place,divisional_chart_factor=1,years=1,months=1,sixty_hours=1,include_antardhasa=True):
    method = 2 # KN Rao Method - Working 1=< Sanjay Rath - yet to be implemented
    jd_at_dob = utils.julian_day_number(dob, tob)
    planet_positions = charts.divisional_chart(jd_at_dob, place, ayanamsa_mode=const._DEFAULT_AYANAMSA_MODE, 
                                               divisional_chart_factor=divisional_chart_factor, years=years, 
                                               months=months, sixty_hours=sixty_hours)
    asc_house = planet_positions[0][1][0] ; seventh_house = (asc_house+6)%12
    dhasa_seed = asc_house
    if method == 1 and asc_house in const.even_signs: # Sanjay Rath Method
        dhasa_seed = 8
        if asc_house in const.fixed_signs:
            dhasa_seed = 6
        elif asc_house in const.dual_signs:
            dhasa_seed = 7
    else:
        dhasa_seed = house.stronger_rasi_from_planet_positions(planet_positions, asc_house, seventh_house)
    dir = 0
    if dhasa_seed in const.even_signs:
        dir = 1
    dhasa_lords = dhasa_order[dhasa_seed][dir]
    #print('dhasa_seed',dhasa_seed,'dhasa_lords',dhasa_lords)
    dhasa_info = []
    start_jd = jd_at_dob
    for dhasa_lord in dhasa_lords:
        dhasa_index = dhasa_lords.index(dhasa_lord)
        duration = _dhasa_duration_kn_rao(planet_positions,dhasa_lord) # 
        if method == 1:
            duration = _dhasa_duration(dhasa_lord)
        #print(house.rasi_names_en[dhasa_lord],duration,'years')
        bhukthis =  [dhasa_lords[(dhasa_index+h)%12] for h in range(12)]
        if include_antardhasa:
            dd = duration/12
            for bhukthi_lord in bhukthis:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                dhasa_info.append((dhasa_lord,bhukthi_lord,dhasa_start,dd))
                start_jd += dd * sidereal_year
        else:
            y,m,d,h = utils.jd_to_gregorian(start_jd)
            dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
            dhasa_info.append((dhasa_lord,dhasa_start,duration))
            start_jd += duration * sidereal_year
    return dhasa_info
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests.mandooka_dhasa_test()


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora import const, utils
from jhora.panchanga import drik
sidereal_year = const.sidereal_year
""" Applicability: Lagna in Taurus/Libra navamsa """

#seed_star = 27 # Poosam
seed_lord = 0
dhasa_adhipathi_list = {0:7,4:9,8:11,3:13,7:15,2:17,6:19,1:21} #  Total 112 years
#dhasa_adhipathi_dict = {0:[27,19,11,3],4:[26,18,10,2],8:[25,17,9,1],3:[24,16,8],7:[23,15,7],2:[22,14,6],6:[21,13,5],1:[20,12,4]}
count_direction = -1 # 1> base star to birth star zodiac -1> base star to birth star antizodiac
def applicability_check(navamsa_planet_positions):
    """ Lagna in Taurus/Libra navamsa """
    navamsa_lagna = navamsa_planet_positions[0][1][0]
    return navamsa_lagna in [1,6]
def _next_adhipati(lord,dirn=1):
    """Returns next lord after `lord` in the adhipati_list"""
    current = list(dhasa_adhipathi_list.keys()).index(lord)
    next_lord = list(dhasa_adhipathi_list.keys())[((current + dirn) % len(dhasa_adhipathi_list))]
    return next_lord
def _get_dhasa_dict(seed_star=27):
    dhasa_dict = {k:[] for k in dhasa_adhipathi_list.keys()}
    nak = seed_star-1
    lord = seed_lord
    lord_index = list(dhasa_adhipathi_list.keys()).index(lord)
    for _ in range(27):
        dhasa_dict[lord].append(nak+1)
        nak = (nak+1*count_direction)%27
        lord_index = (lord_index+1) % len(dhasa_adhipathi_list)
        lord = list(dhasa_adhipathi_list.keys())[lord_index]
    return dhasa_dict
#dhasa_adhipathi_dict = _get_dhasa_dict()

def _maha_dhasa(nak,seed_star=27):
    dhasa_adhipathi_dict = _get_dhasa_dict(seed_star)
    return [(_dhasa_lord, dhasa_adhipathi_list[_dhasa_lord]) for _dhasa_lord,_star_list in dhasa_adhipathi_dict.items() if nak in _star_list][0]
def _antardhasa(dhasa_lord,antardhasa_option=1):
    lord = dhasa_lord
    if antardhasa_option in [3,4]:
        lord = _next_adhipati(dhasa_lord, dirn=1) 
    elif antardhasa_option in [5,6]:
        lord = _next_adhipati(dhasa_lord, dirn=-1) 
    dirn = 1 if antardhasa_option in [1,3,5] else -1
    _bhukthis = []
    for _ in range(len(dhasa_adhipathi_list)):
        _bhukthis.append(lord)
        lord = _next_adhipati(lord,dirn)
    return _bhukthis
def _dhasa_start(jd,place,star_position_from_moon=1,divisional_chart_factor=1,chart_method=1,
                 seed_star=27,dhasa_starting_planet=1):
    y,m,d,fh = utils.jd_to_gregorian(jd); dob=drik.Date(y,m,d); tob=(fh,0,0)
    one_star = (360 / 27.)        # 27 nakshatras span 360°
    from jhora.horoscope.chart import charts,sphuta
    _special_planets = ['M','G','T','I','B','I','P']
    planet_positions = charts.divisional_chart(jd, place, divisional_chart_factor=divisional_chart_factor,
                                               chart_method=chart_method)
    if dhasa_starting_planet in [*range(9)]:
        planet_long = planet_positions[dhasa_starting_planet+1][1][0]*30+planet_positions[dhasa_starting_planet+1][1][1]
    elif dhasa_starting_planet==const._ascendant_symbol:
        planet_long = planet_positions[0][1][0]*30+planet_positions[0][1][1]
    elif dhasa_starting_planet.upper()=='M':
        mn = drik.maandi_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
        planet_long = mn[0]*30+mn[1]
    elif dhasa_starting_planet.upper()=='G':
        gl = drik.gulika_longitude(dob,tob,place,divisional_chart_factor=divisional_chart_factor)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='B':
        gl = drik.bhrigu_bindhu_lagna(jd, place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='I':
        gl = drik.indu_lagna(jd, place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='P':
        gl = drik.pranapada_lagna(jd, place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
        planet_long = gl[0]*30+gl[1]
    elif dhasa_starting_planet.upper()=='T':
        sp = sphuta.tri_sphuta(dob,tob,place,divisional_chart_factor=divisional_chart_factor,chart_method=chart_method)
        planet_long = sp[0]*30+sp[1]
    else:
        planet_long = planet_positions[2][1][0]*30+planet_positions[2][1][1]
    if dhasa_starting_planet==1:
        planet_long += (star_position_from_moon-1)*one_star
    nak = int(planet_long / one_star); rem = (planet_long - nak * one_star)
    lord,res = _maha_dhasa(nak+1,seed_star)          # ruler of current nakshatra
    period = res
    period_elapsed = rem / one_star * period # years
    period_elapsed *= sidereal_year        # days
    start_date = jd - period_elapsed      # so many days before current day
    return [lord, start_date,res]
def get_dhasa_bhukthi(dob,tob,place,include_antardhasa=True,star_position_from_moon=1,use_tribhagi_variation=False,
                      divisional_chart_factor=1,chart_method=1,seed_star=27,dhasa_starting_planet=1,antardhasa_option=1):
    """
        returns a dictionary of all mahadashas and their start dates
        @param dob: Date Struct (year,month,day)
        @param tob: time tuple (h,m,s) 
        @param place: Place as tuple (place name, latitude, longitude, timezone) 
        @param include_antardhasa True/False. Default=True 
        @param divisional_chart_factor Default=1 
            1=Raasi, 9=Navamsa. See const.division_chart_factors for options
        @param chart_method: Default=1, various chart methods available for each div chart. See charts module
        @param star_position_from_moon: 
            1 => Default - moon
            4 => Kshema Star (4th constellation from moon)
            5 => Utpanna Star (5th constellation from moon)
            8 => Adhana Star (8th constellation from moon)
        @param use_tribhagi_variation: False (default), True means dhasa bhukthi duration in three phases 
        @param seed_star 1..27. Default = 27
        @param antardhasa_option:
            1 => dhasa lord - forward (Default)
            2 => dhasa lord - backward
            3 => next dhasa lord - forward
            4 => next dhasa lord - backward
            5 => prev dhasa lord - forward
            6 => prev dhasa lord - backward
        @param dhasa_starting_planet 0=Sun 1=Moon(default)...8=Ketu, 'L'=Lagna
                                    M=Maandi, G=Gulika, T=Trisphuta, B=Bhindu, I=Indu, P=Pranapada
        @return: a list of [dhasa_lord,bhukthi_lord,bhukthi_start]
          Example: [ [7, 5, '1915-02-09'], [7, 0, '1917-06-10'], [7, 1, '1918-02-08'],...]
    """
    _tribhagi_factor = 1.
    _dhasa_cycles = 1
    if use_tribhagi_variation:
        _tribhagi_factor = 1./3.; _dhasa_cycles = int(_dhasa_cycles/_tribhagi_factor)
    jd = utils.julian_day_number(dob, tob)
    dhasa_lord, start_jd,_ = _dhasa_start(jd,place,star_position_from_moon=star_position_from_moon,
                                          divisional_chart_factor=divisional_chart_factor,chart_method=chart_method,
                                          seed_star=seed_star,dhasa_starting_planet=dhasa_starting_planet)
    retval = []
    for _ in range(_dhasa_cycles):
        for _ in range(len(dhasa_adhipathi_list)):
            _dhasa_duration = round(dhasa_adhipathi_list[dhasa_lord]*_tribhagi_factor,2)
            if include_antardhasa:
                bhukthis = _antardhasa(dhasa_lord,antardhasa_option=antardhasa_option)
                _dhasa_duration /= len(bhukthis)
                for bhukthi_lord in bhukthis:
                    y,m,d,h = utils.jd_to_gregorian(start_jd)
                    dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                    retval.append((dhasa_lord,bhukthi_lord,dhasa_start,_dhasa_duration))
                    start_jd += _dhasa_duration * sidereal_year
            else:
                y,m,d,h = utils.jd_to_gregorian(start_jd)
                dhasa_start = '%04d-%02d-%02d' %(y,m,d) +' '+utils.to_dms(h, as_string=True)
                retval.append((dhasa_lord,dhasa_start,_dhasa_duration))
                lord_duration = round(dhasa_adhipathi_list[dhasa_lord]*_tribhagi_factor,2)
                start_jd += lord_duration * sidereal_year
            dhasa_lord = _next_adhipati(dhasa_lord) # dirn=1 for dhasa sequence
    return retval
if __name__ == "__main__":
    from jhora.tests import pvr_tests
    pvr_tests._STOP_IF_ANY_TEST_FAILED = False
    pvr_tests.dwadasottari_test()




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (C) Open Astro Technologies, USA.
# Modified by Sundar Sundaresan, USA. carnaticmusicguru2015@comcast.net
# Downloaded from https://github.com/naturalstupid/PyJHora

# This file is part of the "PyJHora" Python library
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from jhora.panchanga import drik
chart_1_dob = (2000,4,9)
chart_1_tob = (17,55,0)
chart_1_place = drik.Place('unknown',42+30/60,-71-12/60,-4.0)
chart_1_dcf = 1

chart_2_dob = chart_1_dob
chart_2_tob = chart_1_tob
chart_2_place = chart_1_place
chart_2_dcf = 16
# A.B Vajpayee
chart_3_dob = (1926,12,25)
chart_3_tob = (5,12,0)
chart_3_place = drik.Place('unknown',26+14/60,78+10/60,5.5)
chart_3_dcf = 1

chart_4_dob = (1968,3,19)
chart_4_tob = (18,30,0)
chart_4_place = drik.Place('unknown',17+7/60,82+19/60,5.5)
chart_4_dcf = 1

chart_5_dob = (2000,4,19)
chart_5_tob = (19,21,0)
chart_5_place = drik.Place('unknown',42+40/60,-71-8/60,-4.0)
chart_5_dcf = 9
# PV Narasimha Rao
chart_6_dob = (1921,6,21)
chart_6_tob = (12,49,0)
chart_6_place = drik.Place('unknown',18+26/60,79+9/60,5+17/60)
chart_6_dcf = 1
# Ronald Regan
chart_7_dob = (1911,2,6)
chart_7_tob = (2,4,0)
chart_7_place = drik.Place('unknown',41+38/60,-89-47/60,-6.0)
chart_7_dcf = 1

chart_8_dob = (1946,12,2)
chart_8_tob = (6,45,0)
chart_8_place = drik.Place('unknown',38+6/60,15+39/60,1.0)
chart_8_dcf = 1
#Chatrapati Sivaji
chart_9_dob = (1630,2,19)
chart_9_tob = (18,26,0)
chart_9_place = drik.Place('unknown',18+32/60,73+53/60,5.5)
chart_9_dcf = 1 # 9
# Akbar
chart_10_dob = (1542,12,4)
chart_10_tob = (3,39,0)
chart_10_place = drik.Place('unknown',25+19/60,69+47/60,5.0)
chart_10_dcf = 1

chart_11_dob = (1921,6,28)
chart_11_tob = (13,8,0)
chart_11_place = drik.Place('unknown',18+26/60,79+9/60,5.0)
chart_11_dcf = 1

chart_12_dob = (1958,8,16)
chart_12_tob = (7,5,0)
chart_12_place = drik.Place('unknown',43+36/60,-83-53/60,-4.0)
chart_12_dcf = 10

chart_14_dob = (1944,8,20)
chart_14_tob = (7,11,0)
chart_14_place = drik.Place('unknown',18+58/60,72+49/60,5.5)
chart_14_dcf = 3

#Bill Cosby
example_27_dob = (1937,7,12)
example_27_tob = (0,30,0)
example_27_place = drik.Place('unknown',39+57/60,-75-10/60,-5.0)
example_27_dcf = 3
